/*******************************************************************************
* Title: Interactive LED Beer Pong Table                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                         
*                                                                              
* Description:                                                                 
* This project contains all of the files and source code that is required to    
* operate the Interactive LED Beer Pong Table. You can find all of the information
* needed for this project at:     
*
* Instructable    
* http://www.instructables.com/id/EQNIAZFHS9TD02J/
*
* Video:
* https://www.youtube.com/watch?v=_OS11MW9bxU
*                      
*******************************************************************************/

/*************************************************
* Include device file and configuration settings *
*************************************************/
#include "Main_Includes.h"
#include "Globals.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "Delay_Setup.h"
#include "PORT_Setup.h"
#include "ADC_Setup.h"
#include "SPI_Setup.h"
#include "PWM_Setup.h"
#include "Timers_Setup.h"
#include "TLC5940_Setup.h"
#include "HT1632C_Setup.h"
#include "LED_Graphics.h"
#include "IR_Sensors.h"
#include "Pong_Game.h"
#include "nRF24L01_Setup.h"
#include "RF_Controls.h"
#include "Miscellaneous.h"

/*************************************************
*             Configuration Settings             *
*************************************************/
_FOSCSEL(FNOSC_PRIPLL & IESO_ON)
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF & POSCMD_HS)
_FWDT(FWDTEN_OFF & WINDIS_OFF); 
_FPOR(FPWRT_PWR1); 
_FICD(JTAGEN_OFF & ICS_PGD1)

/*******************************************************************************
* Function: main(void)                                                              
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*
* All of the code in the program is ran from inside this function. This        
* function will never terminate as long as the MCU has power applied to it.    
*******************************************************************************/
int main(void)
{ 
  //Initalize PORTs and peripherals 
  PORT_Init();
  
  //Initialize the TLC5940s that control the RGB pods  
  TLC5940_Init();
  
  //Initialize timer interrupts and other peripherals
  Modules_Init();
  
  //Initialize the HT1632C chips
  HT1632_Init();
  
  //Set the default dot correction values for the TLC5940s
  Dot_Correction();
  
  //Run first cycle of grayscale data
  Set_Initial_Grayscale();
  
  //After setting the initial grayscale we can start using the SPI module
  SPI2_Init(); 

  //Reset all global variables to their default states
	Reset_All_Variables();
	
  while(1)
  {  
	  //Cycle through the pod animations
	  Cycle_Pod_Animations();
  }
}

/*******************************************************************************
* Function: Timer1_Interrupt()                                                  
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* Timer1 controls the operation of the RGB pods and updates every 16384uS. This
* interrupt has the highest priority as it is used to control the timing of the
* TLC5940 PWM drivers. 
*******************************************************************************/
void __attribute__((__interrupt__, __auto_psv__)) _T1Interrupt(void)
{	
 //Check to see if there is new data that needs to be updated
 XLAT_Interrupt();

 //Clear the interrupt flag
 _T1IF = 0;
}

/*******************************************************************************
* Function: Timer3_Interrupt()                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* Timer3 controls the fading of the RGB pods (LED_State), handles a global counter
* that is used by various timing routines (count32) and refreshes the LED grid if                                                                              
* there is data that needs to be written to it (LED_Refresh_Grid). Timer3 interrupts
* every 8ms (~120Hz).                                                                             
*******************************************************************************/
void __attribute__((__interrupt__, __auto_psv__)) _T3Interrupt(void)
{
	UINT8 status;
	
  //Increment the global counter
  count32++; 
  
  //Check to see if any RGB pods are fading and update them if they are
  Fade_State();  
  
  //Update the LED grid if there is new data that needs to be written to it
  if (HT1632_UPDATE)
  {
    HT1632_UPDATE = 0;
    LED_Refresh_Grid(LED_data);
  }  			 

  //If there is more text to be scrolled, update the text
	if (SCROLL_ACTIVE)
	  Update_Text();		
	
	//If DIAGNOSE_ERROR is set, an error is present  
	if (DIAGNOSE_ERROR)
	  Display_Error(error_code);	
	
  //Clear the interrupt flag
  _T3IF = 0;
}

/*******************************************************************************
* Function: Timer5_Interrupt()                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This timer has not been implemented yet. It is set to interrupt every 1ms.
* Enable it in Modules_Init() located in PORT_Setup.h if you need it.                                                                                                                                                        
*******************************************************************************/
void __attribute__((__interrupt__, __auto_psv__)) _T5Interrupt(void)
{ 
	//Clear the interrupt flag
  _T5IF = 0;
}
