/*******************************************************************************
* Title: LED_Graphics.c                                                                       
* Version: 1.0                                                                 
* Date: April 23, 2014                                                                         
*                                                                              
* Description:                                                                 
* This file contains various functions that are use to update and control the RGB pods,
* LED grid and LED rings. 
*
* In the diagram below each 'x' represents one LED. This is how the data is packed and
* arranged for the LED grid. For the 2nd HT1632C driver, we only use a 32x4 grid which
* gives our LED grid a total of (32x8 + 32x4 = 32x12) 384 pixels.
*                              
*       16-Bit Data Array 'LED_data[32]'                      
*           0 1 2 3 4 5 ... 31           
*           | | | | | | | | |                             
*       0 - x x x x x x x x x  _                
*       1 - x x x x x x x x x   |                   
*       2 - x x x x x x x x x   |               
*       3 - x x x x x x x x x   |                            
*    B  4 - x x x x x x x x x   | HT1632C Driver Chip #1 (CS1)                                        
*    i  5 - x x x x x x x x x   |                                         
*    t  6 - x x x x x x x x x   |                                                    
*    s  7 - x x x x x x x x x  _|
*                              _                                                                                                                                              
*       8 - x x x x x x x x x   |                                                                                                        
*       9 - x x x x x x x x x   | HT1632C Driver Chip #2 (CS2)                                                                                                        
*      10 - x x x x x x x x x   |                                                                                                        
*      11 - x x x x x x x x x  _|
*                                                                                                  
*******************************************************************************/

#ifndef LED_GRAPHICS_C
#define LED_GRAPHICS_C

#include "Main_Includes.h"
#include "LED_Graphics.h"
#include <string.h>
#include "Miscellaneous.h"
#include "HT1632C_Setup.h"    
#include "Math.h"   
#include "Font_5x7.h"
#include "ADC_Setup.h"
#include "Delay_Setup.h"

/*************************************************
*               Global Variables                 *
*************************************************/
extern volatile UINT8 str_place;
extern volatile UINT8 ltr_place;
extern volatile UINT8 str_length;
extern volatile UINT8 scroll_delay;
extern volatile UINT8 seq[SEQ_AMOUNT];
extern volatile char global_string[64];

extern volatile T16_FLAG FLAG1;
extern volatile UINT16 LEDring[16];
extern volatile UINT16 LED_data[32];
extern volatile UINT16 letter_buffer[6];

extern volatile UINT32 count32;
extern volatile const RGB COLOR[12];

/*******************************************************************************
* Function: Box_Grid_In(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will create a border around the LED grid and make it 1-pixel wider
* at a time until all of the pixels are on, then it will reverse what it just did 
* until all of the pixels are off. It will then repeat the process making a box in
* animation. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                          
*******************************************************************************/
UINT8 Box_Grid_In(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int delay = 5;  
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[20] == 0xFF)
  {
	  seq[20] = 0;
    last_seq = 0xFF;
  }  
  
  //If a new sequence is starting, update it
  if (last_seq != seq[20])
  {  
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[20];
    tmark = count32;
    
    //Clear grid each time a new circle is to be drawn 
    Clear_Grid(); 
    
    //On each sequence increase the border size up to a total of 6-pixels
    if (seq[20] < 7)
	    Draw_Border(seq[20]);
	  
	  //Now reverse what we just did for the next 6 sequences
	  else
	    Draw_Border(12-seq[20]);
    
    //Update the LED grid
    HT1632_UPDATE = 1;
  } 
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[20]++;
    
  //If the animation has completed reset seq[20] to its default state and
  //return a 0  
  if (seq[20] > 12)
  {
    seq[20] = 0xFF;
    return 0;
  }  
  
  //The animation has not completed yet, return a 1
  return 1;  
} 

/*******************************************************************************
* Function: Checkers(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will display a 32x12 checkerboard across the LED grid and alternate
* the state of the LEDs on the checkerboard.  
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                           
*******************************************************************************/         
UINT8 Checkers(void)
{ 
  UINT8 i;
  UINT8 delay = 14;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[16] == 0xFF)
  {
    seq[16] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[16] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[16];
    tmark = count32;
    
    //Display 1st checker pattern on grid
    if (((seq[16]+1) % 2) == 0)
    {
      for (i = 0;i < 32;i++)
      {
        LED_data[i] = 0b101010101010;
        LED_data[++i] = 0b010101010101;
      }  
    }
    
    //Display inverted checker pattern on grid
    else
      Invert_Grid();
    
    //Update the grid
    HT1632_UPDATE = 1;     
  }  
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[16]++;
  
  //If all sequences have been performed, reset seq[x]
  //and return a 1 to indicate that the routine is finished
  if (seq[16] > 19)
  {
    seq[16] = 0xFF;
    return 0;
  }  
  
  //The function has not finished
  return 1;
}	

/*******************************************************************************
* Function: Circle_Out(void)                                                                     
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will create a circle in the middle of the grid and expland it
* outwards until it passed the borders of the grid.  
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                              
*******************************************************************************/
UINT8 Circle_Out(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int delay = 5;  
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[3] == 0xFF)
  {
	  seq[3] = 0;
    last_seq = 0xFF;
  }  
  
  //Loop 16 times, a radius of 16 is a diameter of 32, which is the full width of
  //the grid.
  if (last_seq != seq[3])
  {  
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[3];
    tmark = count32;
    
    //Clear grid each time a new circle is to be drawn 
    Clear_Grid(); 
    
    //Redraw Pong Border
    Draw_Border(1);
    
    //Draw the size of the circle starting from the center of the grid
    Draw_Circle(15,6,seq[3]); 
    
    HT1632_UPDATE = 1;
  } 
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[3]++;
    
  //If the animation has completed reset seq[3] to its default state and
  //return a 0  
  if (seq[3] > 16)
  {
    seq[3] = 0xFF;
    return 0;
  }  
  
  //The animation has not completed yet, return a 1
  return 1;  
}  

/*******************************************************************************
* Function: Clear_Grid(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will clear all of the LED bits in the grid. In order to write these
* new values to the HT1632 drivers one must set HT1632_UPDATE = 1 after this call.  
*******************************************************************************/         
void Clear_Grid(void)
{
 UINT8 i;
 
 //Clear all bits (LEDs)
 for (i = 0;i < 32;i++)
   LED_data[i] = 0x0000;
}    
 
/*******************************************************************************
* Function: Draw_Border(UINT8 width)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will draw a border along the edges of the LED grid. The width of
* border is determined by the variable 'width'. This function will NOT clear the
* grid, instead it will draw a border around whatever is already drawn on the grid.
* If you need the grid cleared before running this function just call Clear_Grid()
* right before you execute it.                                                                            
*******************************************************************************/
void Draw_Border(UINT8 width)
{
  //Determine how wide the border is and turn on the appropriate pixels
  switch (width)
  {
    case 6: Draw_Rect(5,5,22,2); 
    case 5: Draw_Rect(4,4,24,4); 
    case 4: Draw_Rect(3,3,26,6); 
    case 3: Draw_Rect(2,2,28,8); 
    case 2: Draw_Rect(1,1,30,10); 
    case 1: Draw_Rect(0,0,32,12); break;  
  }  
} 
	
/*******************************************************************************
* Function: Corner_Circles(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function is an animation which draws a circle in each corner of the grid.
* The circles then expand across the grid intersecting with one another and 
* making a neat animation. Once the intersected circles pass each other, a circle
* is expanded from the center of the grid to produce a rippling effect. This 
* function will return a 0 if all sequences are completed and the routine is done. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                           
*******************************************************************************/
UINT8 Corner_Circles(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int delay = 5;
  
  int x = 31;
  int y = 11;
  
  //Set variables and the LED grid to a start-up state
  if (seq[5] == 0xFF)
  {
    seq[5] = 0;
    last_seq = 0xFF;
    Clear_Grid();
  } 
  
  //If the seq has changed, update the cycle and change colors
  if (last_seq != seq[5])
  {  
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[5];
    tmark = count32;
    
    //Clear grid for new circle on each sequence lower than 50
    if (seq[5] < 50)
     Clear_Grid(); 
    
    //Re-draw the circles in each corner of the grid
    Draw_Circle(31-x,11-y,seq[5]);
    Draw_Circle(x,11-y,seq[5]);
    Draw_Circle(31-x,y,seq[5]);
    Draw_Circle(x,y,seq[5]);
    
    //Produce the rippling effect by drawing the expanded center circles at 
    //different times in the animation
    if (seq[5] > 21)
     Draw_Circle(15,5,seq[5]-21);
    
    if (seq[5] > 25)
     Draw_Circle(15,5,seq[5]-25);
    
    if (seq[5] > 30)
     Draw_Circle(15,5,seq[5]-30);
    
    if (seq[5] > 36)
     Draw_Circle(15,5,seq[5]-36);
     
    if (seq[5] > 42)
     Draw_Circle(15,5,seq[5]-42);
     
    if (seq[5] > 49)
     Draw_Circle(15,5,seq[5]-49);
    
    if (seq[5] > 54)
    	Fill_Grid();
    
    //Update the LED grid	
    HT1632_UPDATE = 1;	
  } 
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[5]++;
    
  //If all sequences have been performed, reset seq[5]
  //and return a 0 to indicate that the routine is finished
  if (seq[5] > 63)
  {
    seq[5] = 0xFF;
    return 0;
  }  
  
  //Return a 1, indicating that the routine has not ended
  return 1;
}

/*******************************************************************************
* Function: Draw_Circle(UINT8 px, UINT8 py, UINT8 radius)                                                                     
*                                                                              
* Variables:                                                                   
* px -> x-coordinate corresponding to center of circle
* py -> y-coordinate corresponding to center of circle 
* radius -> The radius of the circle                                                                
*                                                                              
* Description:                                                                 
* This function will draw a circle at a specified (x,y) location from the center out.
* Once this routine has been ran, a grid update must be called in order to update
* the new LED data to the grid.                                                                             
*******************************************************************************/         
void Draw_Circle(UINT8 px, UINT8 py, UINT8 radius)
{
 INT8 x;
 INT8 y;
 INT8 xchange;
 INT8 ychange;
 INT8 raderr;
 
 //Set the values for the difference between px and py
 x = radius;
 y = 0;
 
 //Calculate the change that is needed in accrodance with the radius
 xchange = 1 - (radius * 2);
 ychange = 1;
 raderr = 0;
 
 //Using the specified radius, calculate and set all of the pixels in their
 //required quadrants
 while (x >= y)
 {
   LED_Pixel(px+x,py+y,ON);
   LED_Pixel(px-x,py+y,ON);
   LED_Pixel(px-x,py-y,ON);
   LED_Pixel(px+x,py-y,ON);
   LED_Pixel(px+y,py+x,ON);
   LED_Pixel(px-y,py+x,ON);
   LED_Pixel(px-y,py-x,ON);
   LED_Pixel(px+y,py-x,ON);
   
   //Update values to determine if the radius size has been met
   y++;
   raderr += ychange;
   ychange += 2;
   
   //Update x coordinate
   if ((raderr * 2) + xchange > 0)
   {
     x--;
     raderr += xchange;
     xchange += 2;
   }
 }
} 
 
/*******************************************************************************
* Function: Draw_Rect(UINT8 px, UINT8 py, UINT8 sx, UINT8 sy)                                                                    
*                                                                              
* Variables:                                                                   
* px -> The x-coordinate where the rectangle will be drawn from                                                                           
* py -> The y-coordinate where the rectangle will be drawn from                                                                              
* sx -> The horizontal length of the rectangle in pixels                                                                          
* sy -> The vertical length of the rectangle in pixels                                                                          
*                                                                              
* Description:                                                                 
* This function will draw a rectangle starting at (px,py) and the size of the
* rectangle will be set by sx and sy.                                                                            
*******************************************************************************/
void Draw_Rect(UINT8 px, UINT8 py, UINT8 sx, UINT8 sy) 
{
  UINT8 i;
  
  //Adjust the size of the rectangle for the loops below
  sx -= 1;
  sy -= 1;
  
  //Draw the x-pixels along the rows first
  for (i = px;i <= px+sx;i++)
  {
    LED_Pixel(i,py,ON); 
    LED_Pixel(i,py+sy,ON); 
  }    
  
  //Draw the y-pixels along the columns second
  for (i = py;i <= py+sy;i++)
  {
    LED_Pixel(px,i,ON); 
    LED_Pixel(px+sx,i,ON); 
  }  
}       
 
/*******************************************************************************
* Function: Draw_Sine(UINT8 state)                                                                   
*                                                                              
* Variables:                                                                   
* state -> Determines if the sine wave is inverted or not                                                                          
*                                                                              
* Description:                                                                 
* This function will display a sine wave across the LED grid. The sine wave can
* be inverted (0) or display originally (1). Every loop through will modify the
* (x,y) location of the sine wave by one pixel. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                          
*******************************************************************************/
UINT8 Draw_Sine(UINT8 state)   
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
	static float x = 0;
	float dx = 0.3;
	int data[32];
	int amplitude = 5;
  int delay = 5;
	int i;
	
	//Check to see if the program is starting/restarting
  if (seq[6] == 0xFF)
  {
    seq[6] = 0;
    last_seq = 0xFF;
  }  
  
  //If the seq has changed, update the cycle and change colors
  if (last_seq != seq[6])
  {  
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[6];
    tmark = count32;
		
		//Check to see if the inverted sine wave or the original sine wave 
		//has been selected and modify the grid accordingly
		if (state == 0)
	    Fill_Grid();
	  else
	    Clear_Grid();	
	  
	  //Draw a new sine wave across the grid
		for (i = 31;i >= 0;i--)
		{
	  	data[i] = (sin(x*0.7) * amplitude) + 6;
			x += dx;
	    
	    //This will make the sine wave 3-pixels wide; Add or remove more 
	    //pixels to make it wider or thinner
			LED_Pixel(i,data[i]-1,state);
			LED_Pixel(i,data[i],state);
			LED_Pixel(i,data[i]+1,state);
		}
		
		//Update the grid
	  HT1632_UPDATE = 1;
	}
		
	//If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[6]++;
    
  //If all sequences have been performed, reset seq[6] to its default value
  //and return a 0
  if (seq[6] > 31)
  {
    seq[6] = 0xFF;
    return 0;
  } 
  
  //The animation has not completed all sequences, return a 1
  return 1;
}	  
 
/*******************************************************************************
* Function: Dual_Wave(UINT8 state)                                                                   
*                                                                              
* Variables:                                                                   
* state -> Determines if the sine wave is inverted or not                                                                          
*                                                                              
* Description:                                                                 
* This function will display two waves across the LED grid. The waves can be
* inverted (0) or display originally (1). Every loop through will modify the
* (x,y) location of the sine waves by one pixel. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                            
*******************************************************************************/
UINT8 Dual_Wave(UINT8 state)   
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
	static float x = 0;
	static float x2 = 0;
	float dx = 0.3;
	float dx2 = 0.3;
	int data[32];
	int amplitude = 5;
  int delay = 5;
	int i;
	
	//Check to see if the program is starting/restarting
  if (seq[21] == 0xFF)
  {
    seq[21] = 0;
    last_seq = 0xFF;
  }  
  
  //If the seq has changed, update the cycle and change colors
  if (last_seq != seq[21])
  {  
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[21];
    tmark = count32;
		
		//Check to see if the inverted sine wave or the original sine wave 
		//has been selected and modify the grid accordingly
		if (state == 0)
	    Fill_Grid();
	  else
	    Clear_Grid();	
	  
	  //Draw a new sine wave across the grid
		for (i = 31;i >= 0;i--)
		{
	  	data[i] = (sin(x*0.7) * amplitude) + 6;
			x += dx;
	    
	    //This will make the sine wave 3-pixels wide; Add or remove more 
	    //pixels to make it wider or thinner
			LED_Pixel(i,data[i]-1,state);
			LED_Pixel(i,data[i],state);
			LED_Pixel(i,data[i]+1,state);
		}
	  
	  //Draw a new sine wave across the grid
		for (i = 0;i < 32;i++)
		{
	  	data[i] = (sin(x2*0.7) * amplitude) + 6;
			x2 += dx2;
	    
	    //This will make the sine wave 3-pixels wide; Add or remove more 
	    //pixels to make it wider or thinner
			LED_Pixel(i,data[i]-1,state);
			LED_Pixel(i,data[i],state);
			LED_Pixel(i,data[i]+1,state);
		}
		
		//Update the grid
	  HT1632_UPDATE = 1;
	}
		
	//If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[21]++;
    
  //If all sequences have been performed, reset seq[21] to its default value
  //and return a 0
  if (seq[21] > 31)
  {
    seq[21] = 0xFF;
    return 0;
  } 
  
  //The animation has not completed all sequences, return a 1
  return 1;
}	  

/*******************************************************************************
* Function: End_Blast(side)                                                                    
*                                                                              
* Variables:                                                                   
* side -> The side of the table that this function will 'shoot' towards                                                                         
*                                                                              
* Description:                                                                 
* This function will display an animation that shoots a line across the BPT
* and is followed by an arrow that turns on all of the LEDs behind it, it will
* then reach the other side and do an 'exploding' animation once it hits.    
* The function takes one parameter 'side' which will determine what side of
* the table the animation should travel towards. If you pass in the constant
* MASTER_SIDE (1), the animation will travel from the secondary side towards the
* master side and end there. The SECONDARY_SIDE (2) constant will travel from the
* master side towards the secondary side. 
*
* This function has to be looped through with the rest of the code in order to
* keep updating the animation. If the animation has finished or has been passed a 
* value other than 1 or 2, it will simply return a 0. If it is in the process of
* completing an animation, it will return the same value as what had been passed into
* it (a 1 (MASTER_SIDE) or a 2 (SECONDARY_SIDE)). This allows us to keep track of 
* the state of this animation in other functions.                                                
*******************************************************************************/
UINT8 End_Blast(UINT8 side)
{ 
  static UINT32 tmark = 0;
  static UINT8 delay = 4;
  int i;
  INT8 j;
	
	//If this function is restarting, set up the variables
	if (seq[11] == 0xFF)
	{  	
    seq[11] = 0;
    delay = 2;
  }
  
  //If the animation has finished, reset seq[x] and return a 0 to let the
  //program know that it has finished.    
  if (seq[11] > 89)
  {  
    seq[11] = 0xFF;
    return 0;
  }   
  
  //If the master side has been selected, shoot the animation from the 
  //secondary side TOWARDS the master side.     
  if (side == MASTER_SIDE)
  {    	
    //If the seq has changed, update the new sequence
    if (Time_Check(&tmark,delay))
    { 
      //Reset all of the LED grid data     			  			 
      Clear_Grid();
      
      //Start drawing the two lines from the opposite side of the table
      j = 31 - seq[11];
      
      //For each sequence, increase the length of the line by 1 pixel until
      //it reaches the master side			  			 
      for (i = 31;i >= j;i--)
      {
        //Only add to the length of the line once per sequence
        if (j >= 0)
          LED_data[i] = 0x0060;
  		  
  		  //If the rest of the line doesn't need to be updated yet, break
  		  //out of the loop.
  		  else
    		  j = 0xFF; 
  		}	
  		
  		//The next 5 'if' statements just begin drawing circles at
  		//different sequences to provide an 'exploding' effect
  		//Slow the speed in half for these last sequences
  		if (seq[11] > 32)
  		{
        delay = 4;
  		  Draw_Circle(0,5,seq[11] - 32); 
  		}   
  		
  		if (seq[11] > 38)
  		  Draw_Circle(0,5,seq[11] - 37);  
  		
  		if (seq[11] > 42)
  		  Draw_Circle(0,5,seq[11] - 42);  
  		
  		if (seq[11] > 47)
  		  Draw_Circle(0,5,seq[11] - 47);  
  		
  		if (seq[11] > 52)
  		  Draw_Circle(0,5,seq[11] - 52);  
  		
  		//Follow the exploding circles by turning on each row sequentially
  		//behind them.
  		if (seq[11] > 56)
  		{ 
    		//Only update one row per sequence   
  			for (i = 56;i < seq[11];i++)
  		    LED_data[i-56] = 0xFFF; 
  		}
  		
  		//Update to the next sequence 
  		seq[11]++;
  		
  		//Update the LED grid
      HT1632_UPDATE = 1;     
  	}
  	
  	//The animation has not finished yet, return MASTER_SIDE (1)
    return MASTER_SIDE;
  }	
  	
  else if (side == SECONDARY_SIDE)  
  {   	
    //If the seq has changed, update the new sequence
    if (Time_Check(&tmark,delay))
    { 
      //Reset all of the LED grid data       	 
      Clear_Grid();
      
      //Start drawing the two lines from the opposite side of the table
      j = seq[11];

      //For each sequence, increase the length of the line by 1 pixel until
      //it reaches the master side      			  			 
      for (i = 0;i < j;i++)
      {
        //Only add to the length of the line once per sequence
        if (j < 32)
          LED_data[i] = 0x0060;
  		  
  		  //If the rest of the line doesn't need to be updated yet, break
  		  //out of the loop.
  		  else
    		  j = 0xFF;
  	  }	
  		
  		//The next 5 'if' statements just begin drawing circles at
  		//different sequences to provide an 'exploding' effect
  		if (seq[11] > 32)
  		{
        delay = 4;
  		  Draw_Circle(31,5,seq[11] - 32); 
  		}   
  		
  		if (seq[11] > 38)
  		  Draw_Circle(31,5,seq[11] - 37);  
  		
  		if (seq[11] > 42)
  		  Draw_Circle(31,5,seq[11] - 42);  
  		
  		if (seq[11] > 47)
  		  Draw_Circle(31,5,seq[11] - 47);  
  		
  		if (seq[11] > 52)
  		  Draw_Circle(31,5,seq[11] - 52);  
  		  		
  		//Follow the exploding circles by turning on each row sequentially
  		//behind them.
  		if (seq[11] > 56)
  		{    
    		//Only update one row per sequence   
  			for (i = 56;i < seq[11];i++)
  		    LED_data[87-i] = 0xFFF;
  		} 
  		
  		//Update to the next sequence 
  	  seq[11]++;
  	  
  		//Update the LED grid
      HT1632_UPDATE = 1;	
    }	
    
  	//The animation has not finished yet, return SECONDARY_SIDE (2)
    return SECONDARY_SIDE;
  }    	 
  
  //There is no animation to run, reset seq[x] and return a 0
  else
  {
    seq[11] = 0xFF;
    return 0;
  }    
}    

/*******************************************************************************
* Function: Exploding_Circle(void)                                                                     
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will simply draw a circle in the very center of the grid and expand
* it outwards until it expands out of the boundaries of the grid.    
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                            
*******************************************************************************/         
UINT8 Exploding_Circle(void)
{  
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  UINT8 delay = 4;
  UINT8 px;
  UINT8 py;
  
  //Center location of the grid
  px = 15;
  py = 5;
  
  //Reset the function variables to default values
  if (seq[0] == 0xFF)
  {
    seq[0] = 0;
    last_seq = 0xFF;
  }   
    
  //Check to see if the sequence has to be updated
  if (seq[0] != last_seq)
  {
    last_seq = seq[0];
    
    //Clear grid data then write in new data  
    Clear_Grid();
    Draw_Circle(px,py,seq[0]); 
    
    //Update the grid with the new LED data
    HT1632_UPDATE = 1;
   }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[0]++; 
 
  //If all sequences have been performed, reset seq[0] to its default state
  //and return a 0
  if (seq[0] > 15)
  {
    seq[0] = 0xFF;
    return 0;
  }
  
  //Return a 1 to indiccate that the animation has not finished
  return 1;   
}       

/*******************************************************************************
* Function: Fill_Grid(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will set all of the LED bits in the grid. In order to write these
* new values to the HT1632 drivers, we must set HT1632_UPDATE = 1 after this call.                                                                        
*******************************************************************************/         
void Fill_Grid(void)
{
 UINT8 i;
 
 //Set every bit (LED)
 for (i = 0;i < 32;i++)
   LED_data[i] = 0x0FFF;
}	
 
/*******************************************************************************
* Function: Invert_Grid(void)                                                                  
*                                                                              
* Variables:
* N/A                                                                     
*                                                                              
* Description:                                                                 
* This function will invert all of the bits in LED_data. The update grid flag must
* must set after running this in order to display the ivnerted pixels on the grid.                                                                         
*******************************************************************************/
void Invert_Grid(void) 
{
  UINT8 i;
  
  //Invert each bit in the 32 WORD LED_data[x] array
  for (i = 0; i < 32;i++)
    LED_data[i] = ~LED_data[i];
}  

/*******************************************************************************
* Function: LED_Pixel(UINT8 px, UINT8 py, UINT8 state)                                                                     
*                                                                               
* Variables:                                                                    
* px -> The x-coordinate of the pixel
* py -> The y-coordinate of the pixel  
* state -> Determines whether the pixel LED bit will be set to a 1 (ON) or a 0 (OFF)                                                                      
*                                                                               
* Description:                                                                  
* This function will turn the pixel at the (x,y) coordinate off if 'state' is equal to
* 0 or it will turn the pixel on if 'state' is a non-zero value (1).                                                                          
*******************************************************************************/
void LED_Pixel(UINT8 px, UINT8 py, UINT8 state)
{
	//If pixel location is outside of the 32x12 grid, return function  
  if (px > 31 || py > 11)
    return;
  
  //Set the bit to 0 so that the pixel will be turned off when the grid is updated
  if (state == 0)
    BitClr(LED_data[px],py);
    
  //Set the bit to 1 so that the pixel will be turned on when the grid is updated
  else
    BitSet(LED_data[px],py);   
} 

/*******************************************************************************
* Function: LED_Refresh_Grid(UINT16 *data)                                                                      
*                                                                               
* Variables:                                                                    
* *data -> 384 bit variable (64 bytes) that designates one bit to each LED in the grid                                                                         
*                                                                               
* Description:                                                                  
* A user will pass 64 bytes of data into this function and it will transfer the
* data into the HT1632C and clear/set (0 or 1) the specified LEDs.
*
* The diagram in the intro above explains how the grid is laid out, how the data                                                                            
* is packed and which bit corresponds to which LED. The data that is passed into 
* function is called LED_data[32] and each element is 16 bits wide, making 64 bytes.                                                                                                   
*******************************************************************************/
void LED_Refresh_Grid(UINT16 *data)
{  
  INT8 i,j;
  
  //Select the HT1632C #1 and prepare to write data to it
  HT1632_Chip_Select(1);
  HT1632_Write_Bits(HT1632_ID_WR,0x04);
    
  //Set to start writing data at address 0x00
  for (i = 0;i < 7;i++)
  {
    HT1632_DATA = 0;
    HT1632_PULSE(HT1632_WR);
  }  
  
  //Loop through all of the LEDs (bits) in *data that pertain to HT1632C #1 and set
  //the LEDs accordingly
  for (i = 0;i < 32;i++)
  {
    for (j = 0;j < 8;j++)
    {
      HT1632_DATA = (BitTst(data[i],j)) ? 1 : 0;
      HT1632_PULSE(HT1632_WR);
    } 
  }    
  
  //Deselect HT1632C #1, Select #2 and put it into write mode
  HT1632_Chip_Select(-1); 
  HT1632_Chip_Select(2);
  HT1632_Write_Bits(HT1632_ID_WR,0x04);
  
  //Set to start writing data at address 0x00
  for (i = 0;i < 7;i++)
  {
    HT1632_DATA = 0;
    HT1632_PULSE(HT1632_WR);
  }  
    
  //Loop through all of the LEDs (bits) in *data that pertain to HT1632C #2 and set
  //the LEDs accordingly (we only write to half of the addresses on HT1632C #2)
  //Addresses: 0x00, 0x02, 0x04, 0x06, 0x08 ... 0x3E (All even addresses & 0x00)
  for (i = 0;i < 32;i++)
  {
    //Write the last 4 bits of each column to each row in the grid
    for (j = 8;j < 12;j++)
    {
      HT1632_DATA = (BitTst(data[i],j)) ? 1 : 0;
      HT1632_PULSE(HT1632_WR);
    } 
    
    //Write dummy bits to the next 4-bits, as we are not using them and it will then
    //Increment to the next even address that we do use
    HT1632_Write_Bits(0x00,8);
  }   
  
  //Deselect chip #2
  HT1632_Chip_Select(-2);
}     

/*******************************************************************************
* Function: LED_Scoreboard(UINT32 pod_sensors)                                                                  
*                                                                              
* Variables:                                                                   
* pod_sensors -> The data from Check_Sensors() which determines the score                                                                          
*                                                                              
* Description:                                                                 
* This function will take the data containing the IR pod sensor values and
* determine the score for each side of the table. It will then draw a scoreboard
* on the LED grid and display the master sides score and the secondary sides score.
* Each pod counts as 1 point, so each team starts out at 10 points. The first team
* to get their opponent down to 0 wins the match.                                                                    
*******************************************************************************/ 
void LED_Scoreboard(UINT32 pod_sensors)
{
  UINT8 i;
  char temp_str[2];
  char master = 0;
  char secondary = 0;
  static UINT8 last_master = 0xFF;
  static UINT8 last_secondary = 0xFF;
  
  //Add up the score for each side
	for (i = 0;i < 10;i++)
	{
	  master += ((pod_sensors >> i) & 0x01);
	  secondary += ((pod_sensors >> (i+10)) & 0x01);
	} 
  
  //Update the scoreboard if the score has changed since the last loop through
  if (last_master != master || last_secondary != secondary)
  {
    //Update the compare values
	  last_master = master;
	  last_secondary = secondary;
	  
	  //Clear the grid data
	  Clear_Grid();
	  
	  //If the master side has 10 points, adjust the text location and display it	
	  if (master == 10)
		  Set_Text(2,2,"10");
		
		//If the master side is below 10 there is only one charcter. Center the digit.  
		else  
		{
  		//Convert the single numeral to a character (Refer to ASCII chart for help)
			temp_str[0] = master + 0x30;
			temp_str[1] = '\0';
			
			//Set the masters score
		  Set_Text(5,2,temp_str);
		}  
		
	  //If the secondary side has 10 points, adjust the text location and display it	
	  if (secondary == 10)
		  Set_Text(18,2,"10");
		  
		//If the secondary side is below 10 there is only one charcter. Center the digit. 
		else  
		{
  		//Convert the single numeral to a character (Refer to ASCII chart for help)
			temp_str[0] = secondary + 0x30;
			temp_str[1] = '\0';
			
			//Set the secondarys score
		  Set_Text(21,2,temp_str);
		}  
		
		//Draw two dividing lines down the center of the table and draw a 1-pixel border  	
		LED_data[15] = 0xFFF;
		LED_data[16] = 0xFFF;
		Draw_Border(1);
		
		//Update the LED grid
		HT1632_UPDATE = 1;
	}
}    
     
/*******************************************************************************
* Function: Load_Next_Letter(void)                                                                   
*                                                                              
* Variables:   
* N/A                                                                                                                                        
*                                                                              
* Description:                                                                 
* This function will load the next letter from global_string[64] and separate 
* the single letter into separate pieces of data so that we can scroll it across
* the LED grid. This will return a 0 when the end of the string has been reached
* and return a 1 when there are still more characters.                                                                            
*******************************************************************************/ 
UINT8 Load_Next_Letter(void)
{
  UINT8 i;
  UINT16 lookup; 
  
  //Check to see if we are at the end of the string; If we are return a 0
  if (str_place == str_length)
    return 0;
  
  //Calculate the starting location of the letter in the 5x7 font array
  lookup = (global_string[str_place] - 32) * 5;
  
  //This adds more space between letters which provides easier reading
  letter_buffer[5] = 0x00;
  
  //Separate the letter into a buffer
  for (i = 0;i < 5;i++)
    letter_buffer[i] = (font_5x7[lookup+i] << 2);
  
  //Update the string placement for the next letter   
  str_place++;
  
  //Reset letter placement to 0 so that we write the new letter to the grid
  ltr_place =0;
  
  //Return a 1 to indicate that this has completed successfully 
  return 1;
}

/*******************************************************************************
* Function: Read_VU(void)                                                                     
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function can detect the audio intensity in a room. AN2 has an amplifier
* with an electret mic connected up to it. It converts the AC audio to a DC value
* where the MCU can then read it into the ADC in the form of a 10-bit value.                                                                            
*******************************************************************************/ 
UINT16 Read_VU(void)
{
	UINT16 data;
	
	//Prepare to read the VU level
	ADC_CHANNEL(2);
  Delay_us(20); 
  
  //Read and return the VU level
  data = ADC_Read(); 
  
  //Adjust the ADC value in accordance with the sensitivity setting   
  data /= VU_SENSITIVTY;  
  
  //Tweak the ADC value by offsetting it
  data -= VU_OFFSET;
  
  //Return the audio level
  return data;
} 

/*******************************************************************************
* Function: Set_Scrolling_Text(char text[64])                                                                    
*                                                                              
* Variables:                                                                   
* text[64] -> Text that is to be displayed on the grid                                                                          
*                                                                              
* Description:                                                                 
* This function will begin writing text to the LED grid. It will only start the
* first column (ROW31) and then set an interrupt flag. The interrupt routine  
* will then handle the rest of the text and scroll it across the table.                                                                          
*******************************************************************************/    
void Set_Scrolling_Text(char text[64])
{
	UINT8 i;
	
	//Make sure that there is no other text being written to the grid	and that
	//SCROLL_FINISHED is clear so that we can write new data
	if (!SCROLL_ACTIVE & !SCROLL_FINISHED)
	{
  	//Get the length of the string
  	str_length = strlen(text);
  	
  	//If there is no characters or the characters exceed 63, exit the function
  	if (str_length == 0 || str_length > 63)
  	  return;
  	
  	//Copy all of the text into our global text array 'global_string[64]'  
	  for (i = 0;i < str_length;i++)
	    global_string[i] = text[i];
  	
  	//Reset str_place and load the first letter to be written to the grid
  	str_place = 0;
  	Load_Next_Letter();
  	
  	//Clear the grid data
  	Clear_Grid();
  	
  	//Set ROW31 with the first part of the string
  	LED_data[31] = letter_buffer[ltr_place];
  	
  	//Increment this to keep track of what part of the letter we need to show next
  	ltr_place++;
  	
  	//Set the scroll flag so that we know a text scroll operation is in progress
  	SCROLL_ACTIVE = 1;
  	
  	//Update the LED grid
  	HT1632_UPDATE = 1;
  } 	
} 

/*******************************************************************************
* Function: Set_Text(UINT8 px, UINT8 py, char text[12])                                                                  
*                                                                              
* Variables:                                                                   
* px -> Starting x-coordinate where the text is placed                         
* py -> Starting y-coordinate where the text is placed                                                                                      
* text[12] -> The text that is to be displayed                                                                         
*                                                                              
* Description:                                                                 
* This function will write text onto the LED grid array starting at location (px,py).                                                                      
*******************************************************************************/  
void Set_Text(UINT8 px, UINT8 py, char text[12])
{
  UINT8 i;
  UINT8 j;
  UINT8 length;
  UINT16 lookup;
  
  //Get the length of the string
  length = strlen(text);
  	
  //Display each character across the grid
	for (i = 0;i < length;i++)
	{	
		//Track the placement of each letter
		j = px + (i * 6);
		
		//Return the function if the text spans over the limit of the LED grid
		//Each character is in a 5x7 format, ROW26 + 5 Rows for the character
		//will exceed the size of the grid.
		if (j > 26)
		{
			HT1632_UPDATE = 1;
			return;
		}	
				
	  //Find the starting location in the array of Font_5x7[] for the specified character
  	lookup = (text[i] - 32) * 5;
  	
  	//Display the character on the grid
		LED_data[j]   = font_5x7[lookup]   << py;	
		LED_data[j+1] = font_5x7[++lookup] << py;	
		LED_data[j+2] = font_5x7[++lookup] << py;	
		LED_data[j+3] = font_5x7[++lookup] << py;	
		LED_data[j+4] = font_5x7[++lookup] << py;	
		LED_data[j+5] = 0x00;		  	
	}	 
} 	
     
/*******************************************************************************
* Function: Update_Text(void)                                                                   
*                                                                              
* Variables:   
* N/A                                                                                                                                        
*                                                                              
* Description:                                                                 
* This function will update the location of the scrolling text on the LED grid
* and display it. It is set up in the Timer3 interrupt routine.                                                                             
*******************************************************************************/ 
UINT8 Update_Text(void)
{
  UINT8 i;
  static UINT8 end_scrolling = 0;
  static UINT32 tmark = 0;
  
  //Check to see if the specified amount of time has elapsed
  if (Time_Check(&tmark,scroll_delay))
  {
    //Update the counter for the next operation
    tmark = count32;      
	  
	  //Check to make sure that there is more text that needs to be written to the grid
	  if (end_scrolling == 0)
	  {	  
  	  //Shift all of the text on the grid over 1 row  
		  for (i = 0;i < 31;i++)
		    LED_data[i] = LED_data[i+1];
		    
		  //Check to see if we need to load the next letter in the string 
		  if (ltr_place > 5)
		  {
  		  //Check to see if this is the last letter written to the grid
		    if (Load_Next_Letter() == 0)
		    {	
		      //Increment this so that we know to keep scrolling until all of the
		      //string has shifted off of the grid 
		      end_scrolling++;	
		      
		      //Update the grid and return the function     
			    HT1632_UPDATE = 1; 
		      return 1;
		    }   
		    
		    //There are still more letters to load from the string  
		    else
		    {
		      //Add a blank space on the last column
		      LED_data[31] = 0x0000;
		      
		      //Update the grid and return the function   
		      HT1632_UPDATE = 1; 
		      return 1;
		    }   
		  }
	
	    //Update ROW31 with the next part of the letter for the grid
			LED_data[31] = letter_buffer[ltr_place];  
			ltr_place++;    
			 
		  //Update the grid   
			HT1632_UPDATE = 1; 
		}
		
		//All of the text is on the grid, scroll the last of the text until the
		//grid is empty 
		else if (end_scrolling < 32)
		{
  	  //Shift all of the text on the grid over 1 row  
		  for (i = 0;i < 31;i++)
		    LED_data[i] = LED_data[i+1];	
		  
		  //Update the last of the scrolling
		  end_scrolling++;	
		  
		  //Update the grid  
			HT1632_UPDATE = 1;  
	  }     
	  
	  //All of the text has scrolled off of the grid, clear the required bits 
	  else
	  {
	   	SCROLL_ACTIVE = 0;
	   	end_scrolling = 0;
	   	SCROLL_FINISHED = 1;
	   	return 0;
	  }	 
	}
	
	//There is still more text to scroll off of the grid, return a 1
	return 1;  
} 

/*******************************************************************************
* Function: VU_Meter_Bar(void)                                                               
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will light up each row of the LED grid (32 rows) independently 
* according to the sound intensity of the music. If the sound intensity is between 
* ROW0 - ROW26 the color of the RGB pods will be green. If the sound intensity
* is between ROW26 - ROW31 the color of the RGB pods will be yellow and if the
* sound intensity maxes out at ROW31, the color of the RGB pods will be red and
* the LED rings on the table will also be turned on.  
*
* This function is set to allow 32 different audio intensities. This can be modified
* to be higher or lower by adjusting the max value in the function (32 in this case).
*
* I know that the VU meter code can be written much better! I just had to get
* something working quickly. Feel free to modify it and make it more versatile.                                                                     
*******************************************************************************/
void VU_Meter_Bar(void)
{
	UINT8 i;
	UINT8 j;
	static INT16 bar = 0;
	static INT16 new_adc;
	static INT16 old_adc = 0;
	
	//Read the sound itensity/audio response from the ADC module
	new_adc = Read_VU();
  
  //Reset the LED grid data  
	Clear_Grid();
	
	//If the new adc value is less than the adc value from the previous run, only 
	//decrease the value of new_adc by 1. Otherwise the VU Meter will be very jumpy. 
	//This provides a dampening effect.
	if (new_adc < old_adc)
		new_adc = old_adc - 1;
	
	//If the new adc value is more than the adc value from the previous run, only 
	//increase the value of new_adc by 1. Otherwise the VU Meter will be very jumpy. 
	//This provides a dampening effect.
	else if (new_adc > (old_adc))
		new_adc = old_adc + 1; 
		
	//If the adc value is larger than we can allow for the LED grid, update it to the
	//maximum value
	if (new_adc > 33)
		new_adc = 32;
	
	//If the bar is still higher than the audio response on the grid, turn on the bar
	//at its current row 
	if (bar > new_adc && bar > 0)
  {
		LED_data[bar] = 0x0FFF;
	  j++;
	}
	
	//If the audio response is higher than the floating bar, set the bar one row above
	//the audio response 
	else
	{
	  bar = new_adc + 1;
	  j = 0;
	}
	
	//If the bar has been higher than the audio response for the last 7 loops, decrease
	//the height of the bar by one row. This provides a floating effect. The higher that
	//the FLOATING_BAR value is, the longer that it will take for the bar to fall back down.  
	if ((j % FLOATING_BAR)  == 0)
	{
  	//Don't lower the bar if it is already at the bottom of the meter
		if (bar != 0)
	  	 bar--; 	 
	}	   
	
	//Don't allow the grid to update while we are updating the LED data (it may still need
	//to update from the previous loop)  	
	_T3IE = 0; 
	
	//Update the LED data in accordance with the intensity of the audio ADC value
  for (i = 0;i < new_adc-1;i++)
	  LED_data[i] = 0x0FFF;
	 
	//After the LED data has been modified, all the grid to update again  
	_T3IE = 1; 
	
	//If the bar has hit near maximum range, turn on all LED rings and set the RGB
	//pods to the color RED 
	if (bar > 31)
	{
		Set_All_Pods(COLOR[RED]);
		Set_All_Rings(120,1);
	}	
	
	//If the bar is in the caution zone, make sure that the LED rings are off and set
	//the RGB pods to yellow
	else if (bar > 25)
	{
		Set_All_Pods(COLOR[YELLOW]);
		Set_All_Rings(0,1);
	} 
		
	//If the bar is in the safe zone, make sure that the LED rings are off and set
	//the RGB pods to green
	else
	{
		Set_All_Pods(COLOR[GREEN]);
		Set_All_Rings(0,1);
	}	
	
	//Save this adc value for comparison on the next loop
	old_adc = new_adc;
	
	//Update the LED grid
	HT1632_UPDATE = 1;
	
	//Delay a few ms or the VU meter is too jittery. Change this to an interrupt 
	//delay in the future!
	Delay_ms(4);
}	

/*******************************************************************************
* Function: VU_Meter_Circle_Out(void)                                                               
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will draw a circle in the middle of the LED grid and the radius of
* the circle is determined by the audio intensity from the ADC module. If the sound 
* intensity is between values 0 - 26 the color of the RGB pods will be green. If the 
* sound intensity is between values 26 - 30 the color of the RGB pods will be yellow 
* and if the sound intensity maxes out at 31, the color of the RGB pods will be 
* red and the LED rings on the table will also be turned on.  
*
* This function is set to allow 32 different audio intensities. This can be modified
* to be higher or lower by adjusting the max value in the function (32 in this case).
*
* I know that the VU meter code can be written much better! I just had to get
* something working quickly. Feel free to modify it and make it more versatile.                                                                    
*******************************************************************************/
void VU_Meter_Circle_Out(void)
{
	static INT16 new_adc;
	static INT16 old_adc = 0;
	
	//Read the sound itensity/audio response from the ADC module
	new_adc = Read_VU();
  
  //Reset the LED grid data  
	Clear_Grid();
	
	//If the new adc value is less than the adc value from the previous run, only 
	//decrease the value of new_adc by 1. Otherwise the VU Meter will be very jumpy. 
	//This provides a dampening effect.
	if (new_adc < old_adc)
		new_adc = old_adc - 1;
	
	//If the new adc value is more than the adc value from the previous run, only 
	//increase the value of new_adc by 1. Otherwise the VU Meter will be very jumpy. 
	//This provides a dampening effect.
	if (new_adc > (old_adc))
		new_adc = old_adc + 1; 
		
	//If the adc value is larger than we can allow for the LED grid, update it to the
	//maximum value
	if (new_adc > 33)
		new_adc = 32;
	    	
	//Don't allow the grid to update while we are updating the LED data (it may still need
	//to update from the previous loop)  	
	_T3IE = 0; 
	
	//Draw the circle with the radius of new_adc. (radius == 1/2 new_adc)    
	Draw_Circle(15,5,(new_adc>>1));
	  
	//After the LED data has been modified, all the grid to update again  
	_T3IE = 1; 
	
	//If the bar has hit near maximum range, turn on all LED rings and set the RGB
	//pods to the color RED 
	if (new_adc > 31)
	{
		Set_All_Pods(COLOR[RED]);
		Set_All_Rings(120,1);
	}	
	
	//If the new_adc is in the caution zone, make sure that the LED rings are off and set
	//the RGB pods to yellow
	else if (new_adc > 25)
	{
		Set_All_Pods(COLOR[YELLOW]);
		Set_All_Rings(0,1);
	} 
		
	//If the new_adc is in the safe zone, make sure that the LED rings are off and set
	//the RGB pods to green
	else
	{
		Set_All_Pods(COLOR[GREEN]);
		Set_All_Rings(0,1);
	}		
	
	//Save this adc value for comparison on the next loop
	old_adc = new_adc;
	
	//Update the LED grid
	HT1632_UPDATE = 1;
	
	//Delay a few ms or the VU meter is too jittery. Change this to an interrupt 
	//delay in the future!
	Delay_ms(4);
}

/*******************************************************************************
* Function: VU_Meter_Pod_Color(void)                                                               
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will adjust the color of the RGB pods in relation to the sound intensity.
* If there is no sound, the RGB pods will be dimly lit as blue. As the sound intensity
* increases the red channel will get brighter and overtake the blue pods, giving off a
* reddish/pink color. If the audio level hits 31, the LED rings will also turn on.
*
* This function is set to allow 32 different audio intensities. This can be modified
* to be higher or lower by adjusting the max value in the function (32 in this case).
*
* I know that the VU meter code can be written much better! I just had to get
* something working quickly. Feel free to modify it and make it more versatile.                                                                    
*******************************************************************************/
void VU_Meter_Pod_Color(void)
{	
	RGB VU_color;	
	static INT16 new_adc;
	
	//Read the sound itensity/audio response from the ADC module
	new_adc = Read_VU();
		
	//If the adc value is larger than we can allow for the LED grid, update it to the
	//maximum value
	if (new_adc > 33)
		new_adc = 32;
	
	//As the sound intensity increases, increase the bightness of red too 
	//I am using 32 steps for the VU meter. 128 * 32 = 4096, which is the 
	//maximum brightness for the RGB pods
	VU_color.red = (new_adc * 128) - 1; 
	
	//Keep green equal to 0
	VU_color.green = 0;
	
	//Always have a bit of the blue channel turned on
	VU_color.blue = 500;  
	
	//If the sound intensity has hit near maximum range, turn on all LED rings
	//pods to the color RED 
	if (new_adc > 31)
	  Set_All_Rings(40,1);
	  	
	//If the sound intensity is not near max intensity, turn off the LED rings	
	else
	  Set_All_Rings(0,1);
	  
	//Set all pods to the color that correlates with the audio intensity
	Set_All_Pods(VU_color);
	
	//Delay a few ms or the VU meter is too jittery. Change this to an interrupt 
	//delay in the future!
	Delay_ms(3);
}
#endif
