/*******************************************************************************
* Title: Pong_Game.c                                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                        
*                                                                              
* Description:                                                                 
* This file contains the functions that are used to display a pong game animation                                                                             
* across the LED grid.                                                                             
*******************************************************************************/

#ifndef PONG_GAME_C
#define PONG_GAME_C

#include "Main_Includes.h"
#include "HT1632C_Setup.h"
#include "Delay_Setup.h"
#include "LED_Graphics.h"
#include "Miscellaneous.h"
#include "Pong_Game.h"

extern volatile T8_FLAG FLAG1;

/*******************************************************************************
* Function: Pong_Game(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will draw a 1-pixel border around the LED grid, thus providing                                                                             
* a 30x10 grid inside of this border. It will then draw a specified amount of                                                                             
* balls and have them bounce around the table for a specified amount of time                                                                             
* before the program will remove one ball and increase the sizes of the 
* remaining balls. Once it gets to the last ball and finishes all of the steps
* in the program it will call the 'Circle_Out()' function to provide an
* explosion like animation. This function is able to draw a maximum of 10
* balls on the grid at once. The amount that will actually be drawn is equal
* to the value of 'ball_amt'.                                                                          
*******************************************************************************/
UINT8 Pong_Game(void)
{ 
  static PONG_BALL BALL[10];
  static UINT32 tmark = 0;
  static INT16 step = 0;
  static INT16 step2 = 0;
  static UINT8 f_pong = 0;
  static UINT8 ball_amt = BALL_AMOUNT;
  UINT8 i,j;
  
  //This is used to set the direction of each BALL at the start of the program
  //These are random, we don't want the balls to bounce in a pattern with each other
  UINT8 sx[10] = {1,-1,-1,1,1,-1,1,-1,1,-1};
  UINT8 sy[10] = {1,-1,1,-1,-1,-1,1,-1,1,-1};
  
  //This is used to set the xy location of each BALL at the start of the program
  //These are random chosen locations
  UINT8 px[10] = {23,5,26,8,12,2,14,28,19,25};
  UINT8 py[10] = {7,3,2,5,1,6,4,9,8,5};
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (!f_pong)
  { 
    //Initalize all pong balls to their default values  
    for (i = 0;i < ball_amt;i++)
    {
      BALL[i].radius = BALL_RAD;
      BALL[i].speed_x = sx[i];
      BALL[i].speed_y = sy[i];
      BALL[i].new_x = px[i];
      BALL[i].new_y = py[i];
    }
    
    //Set flag so that the function will begin  
    f_pong = 1;
  }
  
  //On first run, this will always prove true. After the first loop through
  //it will only loop through once the time interval has passed and the next
  //step can be executed.
  if (Time_Check(&tmark,GAME_SPEED))
  {
    //Update the sequence, clear the grid and redraw the border but
    //do not update the grid as we will update all of it at once at
    //the end of this function.
    step++;
    Clear_Grid();
    Draw_Border(1);
    
    //After each ball changes locations on the grid 50x times continue to
    //the next step. The value 50 is what determines how long the balls
    //will bounce around on the table before the next step.
    if ((step % 50) == 0)
    {
      //Continue to next step, remove one more ball
      step2++;
      ball_amt--;
      
      //Increase each remaining balls radius by 1 in size
      for (i = 0;i < ball_amt;i++)
      {
        BALL[i].radius = 1 + (step2 >> 1);
        
        //If the ball is on a border and its radius gets increased by 1
        //it will glitch out and get stuck on the border. A quick way to
        //solve this is to move it 1-pixel horizontally and vertically
        //towards the center of the grid every time the radius increases. 
        if (BALL[i].new_x > 16)
        	BALL[i].new_x--; 
        else
        	BALL[i].new_x++; 
        
        if (BALL[i].new_y > 5)
        	BALL[i].new_y--; 
        else
        	BALL[i].new_y++; 
        
      }  
    }    
    
    //Once the time has elapsed for the last ball on the table, perform the
    //'Circle_Out()' animation and clear f_pong to initiate the end of the function
    if (step2 == 7)
    { 
      step2 = 0;    
      
      //Wait for function to finish before continuing
      while (Circle_Out());
      
      //Clear flag
      f_pong = 0;
      ball_amt = BALL_AMOUNT;
      step2 = 0;
      step = 0;
      
      //Return 0 for operation successful
      return 0;
    }  
      
    //Update the direction of the balls and their locations
    for (i = 0;i < ball_amt;i++)
    {
      BALL[i].old_x = BALL[i].new_x;
      BALL[i].old_y = BALL[i].new_y;
      BALL[i].new_x += BALL[i].speed_x;
      BALL[i].new_y += BALL[i].speed_y;
      
      //If the ball has hit a border, change its direction
      if ((BALL[i].new_x - (BALL[i].radius)) < 0)
       BALL[i].speed_x = -BALL[i].speed_x;
       
      //If the ball has hit a border, change its direction
      if ((BALL[i].new_x + (BALL[i].radius)) > GRID_X)
       BALL[i].speed_x = -BALL[i].speed_x;
      
      //If the ball has hit a border, change its direction
      if ((BALL[i].new_y - (BALL[i].radius)) < 0)
       BALL[i].speed_y = -BALL[i].speed_y;
       
      //If the ball has hit a border, change its direction
      if ((BALL[i].new_y + (BALL[i].radius)) > GRID_Y)
       BALL[i].speed_y = -BALL[i].speed_y;
      
      //Draw the ball in its new location
      for (j = 0;j < BALL[i].radius;j++)
       Draw_Circle(BALL[i].new_x,BALL[i].new_y,j);
    }   
    
    //Prepare to update the grid
    HT1632_UPDATE = 1;
  }  
  
  //Return 1 as function hasn't completed
  return 1;
}   
  
#endif
