/*******************************************************************************
* Title: nRF24L01_Setup.c                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                        
*                                                                              
* Description:                                          
* This file contains the functions and commands that are used to set up and control                                                                             
* the operation of nRF24L01 RF Modules.                                                                              
*******************************************************************************/

#ifndef NRF24L01_SETUP_C
#define NRF24L01_SETUP_C

#include "Main_Includes.h"
#include <stdio.h>
#include "SPI_Setup.h"
#include "nRF24L01_Setup.h"
#include "Delay_Setup.h"
  
/*******************************************************************************
* Function: nRF24L01_ARC_CNT(void)                                             
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will return the number of retransmitted packets. The counter   
* is reset back to 0 everytime a new transmission is started.                  
*******************************************************************************/
UINT8 nRF24L01_ARC_CNT(void)
{
 UINT8 arc_cnt;
 
 //Get number of retransmitted packets
 nRF24L01_ReadReg(RF_OBSERVE_TX,&arc_cnt,1);
 
 return (arc_cnt & 0x0F);
} 

/*******************************************************************************
* Function: nRF24L01_Check_ACK(void)                                           
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will check for an ACK response from a receiver, ensuring that  
* a transmission was successful or a failure. (0 == success; non-zero == error)
*******************************************************************************/
UINT8 nRF24L01_Check_ACK(void)
{
  UINT8 status;
      
  //Poll for ACK response; If ACK == received, return success (0); else return ERROR (non-zero)
  while (1)
  {
    //Read STATUS bits
    nRF24L01_ReadReg(RF_STATUS,&status,1);   
    
    //If TX_DS is set, transmission was successful
    if (status & RF_STATUS_TX_DS)
    {
      //Clear TX_DS IRQ and return SUCCESS
      nRF24L01_Clear_IRQ(RF_STATUS_TX_DS);
      return 0x00;  
    } 
    
    //If MAX_RT is set, auto-retransmit has timed-out and not received and ACK, return ERROR
    else if (status & RF_STATUS_MAX_RT)
    {
      //Clear MAX_RT IRQ and return ERROR
      nRF24L01_Clear_IRQ(RF_STATUS_MAX_RT);
      return 0xFD; 
    }        
  } 
}   
 
/*******************************************************************************
* Function: nRF24L01_Check_Ping(UINT8 pipe, UINT8 *data, UINT8 len)             
*                                                                              
* Variables:                                                                   
* pipe -> Data pipe to be pinged (pipe 0 - 5)                                  
* *data -> Pointer to data that is to be pinged back to PTX                    
* len -> The amount of data in bytes that is to be sent                        
*                                                                              
* Description:                                                                 
* This function will check for any received data. If there is any received     
* data, it will load it into a variable, set itself into TX mode, transmit     
* back the same data and then return to RX mode.                               
*******************************************************************************/
UINT8 nRF24L01_Check_Ping(UINT8 pipe, UINT8 *data, UINT8 len)
{  
  UINT8 pipenum;
  UINT8 addr[5]; 
  UINT8 status;
     
  //Get STATUS bits
  nRF24L01_ReadReg(RF_STATUS,&status,1);  
   
  //Check for any new RX data
  if (status & RF_STATUS_RX_DR)
  {
    //Find pipe with data in it
    pipenum = nRF24L01_Get_RX_FIFO_Pipe();
    
    //Find the pipe address of the received data
    if (pipenum > 0)
    {
      //Set TX_ADDR as same as RX_ADDR_P*
      nRF24L01_ReadReg(RF_RX_ADDR_P1,addr,5);
      nRF24L01_ReadReg((RF_RX_ADDR_P0 + pipenum),addr,1);
    }
    
    //Set address to pipe 0
    else
      nRF24L01_ReadReg(RF_RX_ADDR_P0,addr,5);

    //Update the Tx address with the pipe address that we are pinging
    nRF24L01_WriteReg(RF_TX_ADDR,addr,5);
    
    //Load data from RX FIFO
    nRF24L01_Rx(data,len);
    
    //Switch the NRF24L01+ back to TX mode    
    nRF24L01_Mode(RF_TX_MODE,0);
    
    //Transmit back the data
    nRF24L01_Tx(data,len);
      
    //Allow packet enough time to finish transmitting before switching to RX mode
    //If this section of code ever starts to 'hang', add a timeout variable into the loop
    //So it won't loop infinitely. I've never had it hang, so I've never added one.
    do
    { 
      //Get STATUS bits 
      nRF24L01_ReadReg(RF_STATUS,&status,1);
      PORTBbits.RB5 = 0x01;
    }
    while ((status & RF_STATUS_TX_DS) == 0); 
    
    //Switch the NRF24L01+ back to TX mode    
    nRF24L01_Mode(RF_RX_MODE,RF_RX_ACTIVE);
    nRF24L01_Clear_All();
    
    //Return Successful Ping
    return 0x00;
  }   
  
  //Return failed ping; i.e. No Rx data has been received
  return 0xFF;
} 
    
/*******************************************************************************
* Function: nRF24L01_Check_Ping_SB(UINT8 pipe, UINT8 *data, UINT8 len)          
*                                                                              
* Variables:                                                                   
* *data -> Pointer to data that is to be pinged back to PTX                    
* len -> The amount of data in bytes that is to be sent                        
*                                                                              
* Description:                                                                 
* This function will check for any received data. If there is any received     
* data, it will load it into a variable, set itself into TX mode, transmit     
* back the same data and then return to RX mode. Use this function if using    
* Schockburst mode. I have had troubles using this function in the past, when  
* pinging all six pipes, after a while it seems to miss one pipe every loop.   
* I believe it has to do with the ADDRESSES that are set in this routine,      
* however, I cannot spend more time debugging it. Use at your own risk.        
*******************************************************************************/
UINT8 nRF24L01_Check_Ping_SB(UINT8 *data, UINT8 len)
{  
  UINT8 status;
  UINT8 pipenum;
  UINT8 condition; 
  UINT8 addr[5]; 
  UINT8 addr_p0[5];
   
  //Get STATUS bits
  nRF24L01_ReadReg(RF_STATUS,&status,1);  
   
  //Check for any new RX data
  if (status & RF_STATUS_RX_DR)
  {      
    //Find pipe with data in it
    pipenum = nRF24L01_Get_RX_FIFO_Pipe();
        
    //Store the already set up address of pipe 0. This may get changed at one point in the function
    //To receive an auto-ack after a transmit function, it will then be changed back to this stored value
    nRF24L01_ReadReg(RF_RX_ADDR_P0,addr_p0,5); 
       
    //Load data from RX FIFO
    nRF24L01_Rx(data,len);
    
    //If the pipe with data in it is not pipe 0, get the address of the pipe as we will be transmitting
    //Back to the same address on the other RF module
    if (pipenum > 0)
    {
      //Get RX_ADDR_P* ; If confused about this statement, refer to datasheet to see how data pipes
      //1 - 5 are addressed and to gain a better understanding of what I am doing here
      nRF24L01_ReadReg(RF_RX_ADDR_P1,addr,5);
      nRF24L01_ReadReg((RF_RX_ADDR_P0 + pipenum),&addr[0],1);
    }
    
    //If data is on pipe 0, get pipe 0 address
    else
      nRF24L01_ReadReg(RF_RX_ADDR_P0,addr,5);
  
    //Switch the NRF24L01+ back to TX mode    
    nRF24L01_Mode(RF_TX_MODE,0);
    
    //Change TX_ADDR to same address that data was just received on
    nRF24L01_Set_Tx_ACK_Address(addr,5);
    
    //Transmit back the data
    condition = nRF24L01_Tx_SB(data,len);
   
    //Return pipe #0 address back to what it was before Set_Tx_ACK_Address was called
    nRF24L01_Set_Pipe0_Address(addr_p0,RF_ADDR_WIDTH);

    //Switch the NRF24L01+ back to RX mode    
    nRF24L01_Mode(RF_RX_MODE,RF_RX_ACTIVE);
    
    //If transmission does not complete, return error
    if (condition)
      return 0xEF;     
    
    //Return Successful Ping
    return 0x00;
  }   
  
  //Return failed ping; i.e. No Rx data has been received
  return 0xFF;
}
  
/*******************************************************************************
* Function: nRF24L01_Clear_All(void)                                           
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will flush all FIFO buffers and clear all IRQ flags.           
*******************************************************************************/
void nRF24L01_Clear_All(void)
{
  UINT8 status;
  
  //Flush RX and TX FIFOs
  nRF24L01_CMD(RF_FLUSH_TX);
  nRF24L01_CMD(RF_FLUSH_RX);
  
  //Clear all IRQ interrupt flags
  nRF24L01_ReadReg(RF_STATUS,&status,1);
  status |= 0x70;
  nRF24L01_WriteReg(RF_STATUS,&status,1);
}

/*******************************************************************************
* Function: nRF24L01_Clear_IRQ(UINT8 irq)                                      
*                                                                              
* Variables:                                                                   
* irq -> Determines which IRQ is to be cleared                                 
*                                                                              
* Description:                                                                 
* This function will clear the requested interrupt flag. The bit definitions   
* are located in the header file under 'STATUS bits'.                          
*******************************************************************************/
void nRF24L01_Clear_IRQ(UINT8 irq)
{
 UINT8 status;
 
 //Clear the specified IRQ interrupt(s) flag
 nRF24L01_ReadReg(RF_STATUS,&status,1);
 status |= irq;
 nRF24L01_WriteReg(RF_STATUS,&status,1);
} 
  
/*******************************************************************************
* Function: nRF24L01_Clear_PLOS(void)                                          
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will reset the packet loss counter back to 0 in OBSERVE_TX.    
*******************************************************************************/
void nRF24L01_Clear_PLOS(void)
{
 UINT8 channel;
 
 //Reset the packet loss counter
 nRF24L01_ReadReg(RF_CH,&channel,1);
 nRF24L01_WriteReg(RF_CH,&channel,1); 
}
  
/*******************************************************************************
* Function: nRF24L01_CMD(UINT8 name)                                           
*                                                                              
* Variables:                                                                   
* name -> The command name that is to be executed                              
*                                                                              
* Description:                                                                 
* This function will execute a single UINT8 command to the nRF24L01+.           
*******************************************************************************/
void nRF24L01_CMD(UINT8 name)
{
  RF_CSN = 0;
  Delay_us(2);
  SPI_Send(name);
  Delay_us(2);
  RF_CSN = 1; 
  Delay_us(2);
}

/*******************************************************************************
* Function: nRF24L01_Disable_IRQ_Pin(UINT8 irq)                                 
*                                                                              
* Variables:                                                                   
* irq -> Determines which IRQ are enabled                                      
*                                                                              
* Description:                                                                 
* This function will disable the requested interrupt flags on pin IRQ of the     
* nRF24L01+.                                                                   
*******************************************************************************/
void nRF24L01_Disable_IRQ_Pin(UINT8 irq)
{
 UINT8 config;
 
 //Disable the specified interrupts on the IRQ pin
 nRF24L01_ReadReg(RF_CONFIG,&config,1);
 config |= irq;
 nRF24L01_WriteReg(RF_CONFIG,&config,1);
}  

/*******************************************************************************
* Function: nRF24L01_Disable_Pipe(UINT8 pipe)                                   
*                                                                              
* Variables:                                                                   
* pipe -> Data pipe to be disabled (pipe 0 - 5)                                
*                                                                              
* Description:                                                                 
* This function will disable the specified data pipe.                          
*******************************************************************************/
void nRF24L01_Disable_Pipe(UINT8 pipe)
{
 UINT8 en_rxaddr;
 
 //Change pipe# to corresponding bit# in EN_RXADDR register
 pipe = (1 << pipe); 
 
 //Get current pipes setup
 nRF24L01_ReadReg(RF_EN_RXADDR,&en_rxaddr,1);
 
 //Disable specified pipe
 en_rxaddr &= ~pipe;
 nRF24L01_WriteReg(RF_EN_RXADDR,&en_rxaddr,1);
}

/*******************************************************************************
* Function: nRF24L01_Enable_IRQ_Pin(UINT8 irq)                                  
*                                                                              
* Variables:                                                                   
* irq -> Determines which IRQ are enabled                                      
*                                                                              
* Description:                                                                 
* This function will enable the requested interrupt flags on pin IRQ of the      
* nRF24L01+.                                                                   
*******************************************************************************/
void nRF24L01_Enable_IRQ_Pin(UINT8 irq)
{
 UINT8 config;
 
 //Enable the specified interrupts on the IRQ pin
 nRF24L01_ReadReg(RF_CONFIG,&config,1);
 config &= ~irq;
 nRF24L01_WriteReg(RF_CONFIG,&config,1);
}   

/*******************************************************************************
* Function: nRF24L01_Enable_Pipe(UINT8 pipe, UINT8 aa_mode)                     
*                                                                              
* Variables:                                                                   
* pipe -> Data pipe to be enabled (pipe 0 - 5)                                 
* aa_mode -> Auto-Ack Mode (Enable == 1, Disable == 0)                         
*                                                                              
* Description:                                                                 
* This function will enable the specified data pipe. The user also has to set  
* whether the data pipe will use the Auto-Ack feature or not.                  
*******************************************************************************/
void nRF24L01_Enable_Pipe(UINT8 pipe, UINT8 aa_mode)
{
 UINT8 en_rxaddr; 
 UINT8 en_aa; 
 
 //Change pipe# to corresponding bit# in EN_RXADDR register
 pipe = (1 << pipe); 
 
 //Get current pipes and auto-ack setup
 nRF24L01_ReadReg(RF_EN_RXADDR,&en_rxaddr,1);
 nRF24L01_ReadReg(RF_EN_AA,&en_aa,1);
 
 //Enable specified pipe
 en_rxaddr |= pipe;
 nRF24L01_WriteReg(RF_EN_RXADDR,&en_rxaddr,1);
 
 //If Auto-Ack is chosen, enable Auto-Ack on corresponding pipe
 if (aa_mode)
 {
  en_aa |= pipe;
  nRF24L01_WriteReg(RF_EN_AA,&en_aa,1);
 }
 
 //Disable Auto-Ack on corresponding pipe
 else
 {
  en_aa &= ~pipe;
  nRF24L01_WriteReg(RF_EN_AA,&en_aa,1);
 }  
}  

/*******************************************************************************
* Function: nRF24L01_Flush_FIFOs(void)                                         
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will flush both RX and TX FIFOs.                               
*******************************************************************************/
void nRF24L01_Flush_FIFOs(void)
{
 //Flush RX and TX FIFOs
 nRF24L01_CMD(RF_FLUSH_TX);
 nRF24L01_CMD(RF_FLUSH_RX);
} 
  
/*******************************************************************************
* Function: nRF24L01_Get_RX_FIFO_Pipe(void)                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will read STATUS register and mask off the RX_P_NO bits, thus  
* returning the pipe # which the data in RX FIFO is addressed to.              
*******************************************************************************/ 
UINT8 nRF24L01_Get_RX_FIFO_Pipe(void)
{
  UINT8 status;
  UINT8 pipe;
  
  //Get RX_P_NO bits in STATUS
  nRF24L01_ReadReg(RF_STATUS,&status,1);
  
  //Mask off unused bits and shift bits left by 1, giving us the pipe #
  pipe = (status & 0x0E) >> 1;
  
  return pipe;
} 

/*******************************************************************************
* Function: nRF24L01_Init(UINT8 config,                                              
                          UINT8 rf_mode,                                                                           
                          UINT8 en_aa,                                               
                          UINT8 en_rxaddr,                                                         
                          UINT8 setup_aw,                                                   
                          UINT8 setup_retr,                                                  
                          UINT8 channel,                                                                 
                          UINT8 setup)                                                   
* Variables:                                                                   
* config -> Contains CONFIG register settings                                  
* rf_mode -> Sets nRF24L01+ into TX or RX mode (TX = 0; RX = 1)                
* en_aa -> Sets which pipes are enabled for auto-ack                           
* en_rxaddr -> Sets which pipes are enabled for data reception                 
* setup_aw -> Sets the address width for the nRF24L01+                         
* setup_retr -> Sets maximum number or auto-retransmits and delay duration     
* channel -> Sets the RF channel to operate on                                 
* setup -> Contains RF_SETUP register settings                                 
*                                                                              
* Description:                                                                 
* This function will fully initialize the nRF24L01+ to operate as specified by 
* the user. Features such as Shockburst, Enhanced Shockburst, RF Channels, etc,
* are all setup and initialized here.                                          
*******************************************************************************/
void nRF24L01_Init(UINT8 config, 
                   UINT8 rf_mode, 
                   UINT8 en_aa, 
                   UINT8 en_rxaddr, 
                   UINT8 setup_aw, 
                   UINT8 setup_retr,
                   UINT8 channel,
                   UINT8 setup)
{ 
  UINT8 data[5];
  
  //Set pins to default state, allow time to settle before initializing module
  RF_CE = 0;
  RF_CSN = 1;  
  Delay_us(10);
  
  //Set up EN_AA register
  data[0] = en_aa;
  nRF24L01_WriteReg(RF_EN_AA,data,1);
  
  //Set up EN_RXADDR register
  data[0] = en_rxaddr;
  nRF24L01_WriteReg(RF_EN_RXADDR,data,1);
  
  //Set up RF_SETUP register
  data[0] = setup_aw;
  nRF24L01_WriteReg(RF_SETUP_AW,data,1);
  
  //Set up SETUP_RETR register
  data[0] = setup_retr;
  nRF24L01_WriteReg(RF_SETUP_RETR,data,1);
  
  //Set up RF_CH register
  data[0] = channel;
  nRF24L01_WriteReg(RF_CH,data,1);
  
  //Set up RF_SETUP register
  data[0] = setup;
  nRF24L01_WriteReg(RF_SETUP,data,1);
  
  //Set CONFIG<PWR_UP> bit, allow time for module to power up
  config |= RF_CONFIG_PWR_UP;
  nRF24L01_WriteReg(RF_CONFIG,&config,1);
  Delay_ms(2);
  
  //Set CONFIG<PWR_UP> bit, allow time for module to power up
  config |= RF_CONFIG_PWR_UP;
  nRF24L01_WriteReg(RF_CONFIG,&config,1);
  Delay_ms(2);
 
  //Flush any old data in RX and TX FIFOs 
  nRF24L01_Flush_FIFOs();
 
  //Set CE according to whether module is in RX or TX mode
  if (config & RF_CONFIG_PRIM_RX)
    RF_CE = 1;
    
  else 
    RF_CE = 0;
} 

/*******************************************************************************
* Function: nRF24L01_Mode(UINT8 mode, UINT8 active)                             
*                                                                              
* Variables:                                                                   
* mode -> Sets the nRF24L01+ into RX or TX mode                                
* active -> Sets the nRF24L01+ in active mode (1) or standby mode (0)          
*                                                                              
* Description:                                                                 
* This function will set the nRF24L01+ into RX mode or TX mode. The 'active'   
* variable in RX mode will set the module into active or standby mode, this    
* is not applicable to TX mode. TX mode == 0, RX mode == 1.                    
*******************************************************************************/
void nRF24L01_Mode(UINT8 mode, UINT8 active)
{
 UINT8 config;
  
 //Get CONFIG settings
 nRF24L01_ReadReg(RF_CONFIG,&config,1);
 
 //Set module up in RX mode
 if (mode)
 {
   //Set PRIM_RX bit in CONFIG; Enable PRX
   config |= RF_CONFIG_PRIM_RX;
   nRF24L01_WriteReg(RF_CONFIG,&config,1);
   Delay_ms(2);  
   
   //If nRF24L01+ is in RX mode and active, set CE. Otherwise should be in standby-I, clear CE  
   if (active)
     RF_CE = 1;
        
   else
     RF_CE = 0;
 }  
 
 //Set module up in TX mode
 else
 {  
   //Clear PRIM_RX bit in CONFIG; Enable PTX
   config &= (~RF_CONFIG_PRIM_RX);
   nRF24L01_WriteReg(RF_CONFIG,&config,1);
   Delay_ms(2);
   
   RF_CE = 0;
 }  
}    
  
/*******************************************************************************
* Function: nRF24L01_Ping_Pipe(UINT8 pipe, UINT8 *data, UINT8 len)              
*                                                                              
* Variables:                                                                   
* pipe -> Data pipe to be pinged (pipe 0 - 5)                                  
* *data -> Pointer to data that is to be pinged to PRX                         
* len -> The amount of data in bytes that is to be sent                        
*                                                                              
* Description:                                                                 
* This function will send a specified amount of data to a pipe on the PRX, it  
* will then wait to receive back data from the PRX. Once data is received, it  
* will compare that with the original data and determine if it is a copy, thus 
* telling us that the ping was successful. If the function times out before    
* any data is received, it will report an error.                               
*******************************************************************************/  
UINT8 nRF24L01_Ping_Pipe(UINT8 pipe, UINT8 *data, UINT8 len)
{
  UINT16 i = 0;
  UINT8 en_rxaddr;
  UINT8 status;
  UINT8 config;
  
 //Change pipe# to corresponding bit# in EN_RXADDR register
  pipe = (1 << pipe);
  
  //Store which data pipes are enabled
  nRF24L01_ReadReg(RF_EN_RXADDR,&en_rxaddr,1); 
  
  //Return error if specified data pipe is not enabled
  if ((pipe & en_rxaddr) == 0)
     return 0xFC;
  
  //Get CONFIG settings
  nRF24L01_ReadReg(RF_CONFIG,&config,1);
  
  //If nRF24L01+ is in RX mode, change to TX mode
  if (config & RF_CONFIG_PWR_UP)
    nRF24L01_Mode(RF_TX_MODE,0);
 
  //Transmit packet of data to PRX
  nRF24L01_Tx(data,len); 
  
  //Allow packet enough time to finish transmitting before switching to RX mode
  do
  {  
   nRF24L01_ReadReg(RF_STATUS,&status,1);
  }
  while ((status & RF_STATUS_TX_DS) == 0);  
  
  //Set PTX into RX mode and wait for ping response
  nRF24L01_Mode(RF_RX_MODE,RF_RX_ACTIVE);
  
  //Loop for a specified amount of time waiting for a response from PRX
  //If no repsonse comes, operation will time-out
  do
  {
    nRF24L01_ReadReg(RF_STATUS,&status,1);
    
    //Check RX_DR flag to see if data has been received
    if (status & RF_STATUS_RX_DR)
    {
      //Retrieve Rx data in RX FIFO
      nRF24L01_Rx(data,len); 
      
      //Switch the NRF24L01+ back to TX mode    
      nRF24L01_Mode(RF_TX_MODE,0);
      return 0x00;
    } 
    
    i++;
  }
  while (i < 3000); 
      
  //Switch the NRF24L01+ back to TX mode    
  nRF24L01_Mode(RF_TX_MODE,0);
      
  //Function timed-out, no response from PRX, return ERROR  
  return 0xFB;  
}
 
/*******************************************************************************
* Function: nRF24L01_Ping_Pipe_SB(UINT8 pipe, UINT8 *data, UINT8 len)           
*                                                                              
* Variables:                                                                   
* pipe -> Data pipe to be pinged (pipe 0 - 5)                                  
* *data -> Pointer to data that is to be pinged to PRX                         
* len -> The amount of data in bytes that is to be sent                        
*                                                                              
* Description:                                                                 
* This function will send a specified amount of data to a pipe on the PRX, it  
* will then wait to receive back data from the PRX. Once data is received, it  
* will compare that with the original data and determine if it is a copy, thus 
* telling us that the ping was successful. If the function times out before    
* any data is received, it will report an error. Use this function if using    
* Shockburst mode.                                                             
*******************************************************************************/  
UINT8 nRF24L01_Ping_Pipe_SB(UINT8 pipe, UINT8 *data, UINT8 len)
{
  UINT8 config;
  UINT8 status;
  UINT8 condition;
  UINT8 addr[5];
  UINT8 addr_p0[5]; 
  UINT16 i = 0;
  
  //Get CONFIG settings
  nRF24L01_ReadReg(RF_CONFIG,&config,1);
  
  //If nRF24L01+ is in RX mode, change to TX mode
  if (config & RF_CONFIG_PWR_UP)
      nRF24L01_Mode(RF_TX_MODE,0);
    
  //If the pipe with data in it is not pipe 0, get the address of the pipe as we will be transmitting
  //Back to the same address on the other RF module
  if (pipe > 0)
  {
    //Get RX_ADDR_P* ; If confused about this statement, refer to datasheet to see how data pipes
    //1 - 5 are addressed and to gain a better understanding of what I am doing here
    nRF24L01_ReadReg(RF_RX_ADDR_P1,addr,5);
    nRF24L01_ReadReg((RF_RX_ADDR_P0 + pipe),&addr[0],1);
  }
  
  //If data is on pipe 0, get pipe 0 address
  else
    nRF24L01_ReadReg(RF_RX_ADDR_P0,addr,5);
     
  //Store the already set up address of pipe 0. This may get changed at one point in the function
  //To receive an auto-ack after a transmit function, it will then be changed back to this stored value
  nRF24L01_ReadReg(RF_RX_ADDR_P0,addr_p0,5);
      
  //Set TX_ADDR and PO ACK address
  nRF24L01_Set_Tx_ACK_Address(addr,5);   
   
  //Send data to Receiver 
  condition = nRF24L01_Tx_SB(data,len);
  
  //Return pipe #0 address back to what it was before Set_Tx_ACK_Addr was called
  nRF24L01_Set_Pipe0_Address(addr_p0,5);
  
  //If no ACK is received, PRX did not receive data, return error
  if (condition)
  {
    //Clear all IRQ interrupts and flush TX FIFOs
    nRF24L01_Clear_All();
    return condition;
  }
  
  //Ready the nRF24L01+ to receive the pinged packet, switch to RX mode
  nRF24L01_Mode(RF_RX_MODE,RF_RX_ACTIVE);

  //Loop for a specified amount of time waiting for a response from PRX
  //If no repsonse comes, operation will time-out
  do
  {
    nRF24L01_ReadReg(RF_STATUS,&status,1);
    
    //Check RX_DR flag to see if data has been received
    if (status & RF_STATUS_RX_DR)
    {   
      //Retrieve Rx data in RX FIFO
      nRF24L01_Rx(data,len); 
      
      //Switch the NRF24L01+ back to TX mode; Set the RX address of P0 back to its previous address   
      nRF24L01_Mode(RF_TX_MODE,0);

      return 0x00;
    } 
    i++;
  }
  while (i < 10000); 

  //Switch the NRF24L01+ back to TX mode; Set the RX address of P0 back to its previous address    
  nRF24L01_Mode(RF_TX_MODE,0);
  
  //Timeout waiting for response  
  return 0xFA;       
}  
  
/*******************************************************************************
* Function: nRF24L01_PLOS_CNT(void)                                            
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will return the number of lost packets. It is overflow         
* protected at 15 lost packets. To clear it back to 0, the user must write to  
* the RF_CH register.                                                          
*******************************************************************************/
UINT8 nRF24L01_PLOS_CNT(void)
{
 UINT8 plos_cnt;
 
 nRF24L01_ReadReg(RF_OBSERVE_TX,&plos_cnt,1);
 
 return (plos_cnt >> 4); 
}

/*******************************************************************************
* Function: nRF24L01_PowerDown(void)                                           
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will power down the nRF24L01+, saving power.                   
*******************************************************************************/
void nRF24L01_PowerDown(void)
{
  UINT8 config;
  
  //Get CONFIG settings 
  nRF24L01_ReadReg(RF_CONFIG,&config,1);

  //If nRF24L01+ is already powered down, exit function
  if (!(config & RF_CONFIG_PWR_UP))
    return;
   
  //Power down the nRF24L01+, 
  config &= (~RF_CONFIG_PWR_UP);
  nRF24L01_WriteReg(RF_CONFIG,&config,1);
  
  RF_CE = 0;
}  

/*******************************************************************************
* Function: nRF24L01_PowerUp(UINT8 mode)                                        
*                                                                              
* Variables:                                                                   
* mode -> Sets module in TX or RX mode (TX mode = 0; RX mode = 1)              
*                                                                              
* Description:                                                                 
* This function will power up the nRF24L01 and ready it to receive or transmit.
*******************************************************************************/
void nRF24L01_PowerUp(UINT8 active)
{
  UINT8 config;
  
  //Get CONFIG settings
  nRF24L01_ReadReg(RF_CONFIG,&config,1);
  
  //If nRF24L01+ is already powered up, exit function
  if (config & RF_CONFIG_PWR_UP)
    return;
  
  //Power up the nRF24L01+, allow it 2ms to fully wake up  
  config |= RF_CONFIG_PWR_UP;  
  nRF24L01_WriteReg(RF_CONFIG,&config,1);
  Delay_ms(2);
  
  //If nRF24L01+ is in TX mode, set CE into TX mode
  if ((config & RF_CONFIG_PRIM_RX) == 0)
    RF_CE = 0;
  
  //If nRF24L01+ is in RX mode and active, set CE. Otherwise it is inactive, clear CE  
  else
  {
    if (active)
      RF_CE = 1;
      
    else
      RF_CE = 0;
  }    
} 

/*******************************************************************************
* Function: nRF24L01_ReadReg(UINT8 reg, UINT8 *data, UINT8 len)                   
*                                                                              
* Variables:                                                                   
* reg -> Contains the register address that is to be read                      
* *data -> Pointer to data that is to be read from 'reg'                       
* len -> The amount of data to read as pointed to by '*data'                   
*                                                                              
* Description:                                                                 
* This function will read a user specified amount of data from the specified   
* register in the nRF24L01+. The function will return the STATUS register.     
*******************************************************************************/
void nRF24L01_ReadReg(UINT8 reg, UINT8 *data, UINT8 len)
{
  UINT8 i;
  
  RF_CSN = 0; //Enable nRF24L01+
  Delay_us(2);
  
  SPI_Send(RF_READ_REG | reg); //Send READ commmand and register address
  Delay_us(2);
  
  //Begin reading specified amount of data
  for (i = 0;i < len;i++)
  {
    data[i] = SPI_Send(0x00);
    Delay_us(2);
  }  
  
  RF_CSN = 1; //Disable nRF24L01+
  Delay_us(2);
}

/*******************************************************************************
* Function: nRF24L01_ReTransmit(void)                                          
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will retransmit the last payload that was loaded into the TX   
* FIFO.                                                                        
*******************************************************************************/
void nRF24L01_ReTransmit(void)
{ 
 //Tell the nRF24L01 to reuse the last payload in the TX FIFO and transmit it
 nRF24L01_CMD(RF_REUSE_TX_PL);
 nRF24L01_Transmit();
}

/*******************************************************************************
* Function: nRF24L01_Rx(UINT8 *data, UINT8 len)                                 
*                                                                              
* Variables:                                                                   
* *data -> Data that is to be loaded from RX FIFO and received                 
* len -> The amount of data to be loaded from the RX FIFO                      
*                                                                              
* Description:                                                                 
* This function will read out data from the RX FIFO. Each call will read one   
* packet from the RX FIFO.                                                     
*******************************************************************************/
UINT8 nRF24L01_Rx(UINT8 *data, UINT8 len)
{
  UINT8 pipe;
  
  //Find pipe with data in it
  pipe = nRF24L01_Get_RX_FIFO_Pipe();  
   
  //If RX FIFO is empty, return with error  
  if (pipe == 0x07)
       return pipe;  
  
  //Disable PRX from receiving packets while retrieving data from RX FIFO   
  RF_CE = 0;
  nRF24L01_ReadReg(RF_RD_RX_PLOAD,data,len);
  RF_CE = 1;
  
  //Clear IRQ interrupt
  nRF24L01_Clear_IRQ(RF_STATUS_RX_DR);
  
  //Return successfull reception
  return pipe; 
}

/*******************************************************************************
* Function: nRF24L01_Set_Pipe0_Address(UINT8 *rx_addr_p0, UINT8 addr_width)     
*                                                                              
* Variables:                                                                   
* *rx_addr_p0 -> Sets the specified address for RX_ADDR_P0                     
* addr_width -> Sets the address width; Can be from 3 - 5 bytes                
*                                                                              
* Description:                                                                 
* This function will only set the address of pipe 0. This function will mainly 
* be used with the Shockburst function. When Shockburst receives an ACK, it    
* always receives the ACK on pipe 0, therefore the user must change RX_ADDR_P0 
* to match the TX_ADDR in order to rexeive the ACK. The function used to do    
* that is 'Set_Tx_ACK_Address'. In order to change pipe 0 back to its original 
* address before 'Set_Tx_ACK_Address', the user can use this function.         
*******************************************************************************/
void nRF24L01_Set_Pipe0_Address(UINT8 *rx_addr_p0, UINT8 addr_width)
{
  UINT8 data[5];
  
  //Set RX_ADDR_P0 to user specified address
  if (rx_addr_p0 != NULL)  
    nRF24L01_WriteReg(RF_RX_ADDR_P0,rx_addr_p0,addr_width); 
    
  //Set RX_ADDR_P0 to default address (0xE7E7E7E7E7)
  else
  {
    data[0] = RF_RX_ADDR_P0_B0_DV;
    data[1] = RF_RX_ADDR_P0_B1_DV;
    data[2] = RF_RX_ADDR_P0_B2_DV;
    data[3] = RF_RX_ADDR_P0_B3_DV;
    data[4] = RF_RX_ADDR_P0_B4_DV;
    
    nRF24L01_WriteReg(RF_RX_ADDR_P0,data,addr_width); 
  } 
}

/*******************************************************************************
* Function: nRF24L01_Set_RF_Channel(UINT8 channel)                              
*                                                                              
* Variables:                                                                   
* channel -> Sets the frequency channel which the nRF24L01+ operates on        
*                                                                              
* Description:                                                                 
* This function will set the frequency channel that the nRF24L01+ operates on. 
* Writing to this register will automatically clear PLOS_CNT located in the    
* OBSERVE_TX register.                                                         
*******************************************************************************/
void nRF24L01_Set_RF_Channel(UINT8 channel)
{
 nRF24L01_WriteReg(RF_CH,&channel,1); 
}
  
/*******************************************************************************
* Function: nRF24L01_SetRxAddress(UINT8 *rx_addr_p0, 
                                  UINT8 *rx_addr_p1, 
                                  UINT8 addr_width, 
                                  UINT8 *rx_pw)
*                                                                              
* Variables:                                                                   
* *rx_addr_p0 -> Sets the specified address for RX_ADDR_P0                     
* *rx_addr_p1 -> Sets the specified address for RX_ADDR_P1 (See Description)   
* addr_width -> Sets the address width; Can be from 3 - 5 bytes                
* *rx_pw -> Sets the payload size for pipes 0 - 5; Const defined in header file
*                                                                              
* Description:                                                                 
* This function will set up the addresses for the RX datapipes. RX_ADDR_P0 is  
* independent from pipes 1 - 5. Pipes 1 - 5 share the same address except for  
* the LSB of the address. Each LSB is incremented by one in order from         
* smallest to largest. If (*rx_addr == NULL), default addresses will be used.  
*******************************************************************************/
void nRF24L01_SetRxAddress(UINT8 *rx_addr_p0, 
                           UINT8 *rx_addr_p1, 
                           UINT8 addr_width, 
                           UINT8 *rx_pw)
{
  UINT8 data[5];
  
  //Set RX_ADDR_P0 to user specified address
  if (rx_addr_p0 != NULL)  
    nRF24L01_WriteReg(RF_RX_ADDR_P0,rx_addr_p0,addr_width); 
    
  //Set RX_ADDR_P0 to default address (0xE7E7E7E7E7)
  else
  {
    data[0] = RF_RX_ADDR_P0_B0_DV;
    data[1] = RF_RX_ADDR_P0_B1_DV;
    data[2] = RF_RX_ADDR_P0_B2_DV;
    data[3] = RF_RX_ADDR_P0_B3_DV;
    data[4] = RF_RX_ADDR_P0_B4_DV;
    
    nRF24L01_WriteReg(RF_RX_ADDR_P0,data,addr_width); 
  } 
  
  //Set RX_ADDR_P1 to user specified address
  if (rx_addr_p1 != NULL)  
  {
    nRF24L01_WriteReg(RF_RX_ADDR_P1,rx_addr_p1,addr_width); 
    data[0] = rx_addr_p1[0];
  }  
    
  //Set RX_ADDR_P1 to default address (0xC2C2C2C2C2)
  else
  {
    data[0] = RF_RX_ADDR_P1_B0_DV; //Set RX_ADDR_P1 to default address (0xC2C2C2C2C2)
    data[1] = RF_RX_ADDR_P1_B1_DV;
    data[2] = RF_RX_ADDR_P1_B2_DV; 
    data[3] = RF_RX_ADDR_P1_B3_DV; 
    data[4] = RF_RX_ADDR_P1_B4_DV; 
    
    nRF24L01_WriteReg(RF_RX_ADDR_P1,data,addr_width); 
  }  
  
  //Increment each LSB for each pipe and set up the address
  data[0] += 1;
  nRF24L01_WriteReg(RF_RX_ADDR_P2,data,1); 
  data[0] += 1;
  nRF24L01_WriteReg(RF_RX_ADDR_P3,data,1);
  data[0] += 1;
  nRF24L01_WriteReg(RF_RX_ADDR_P4,data,1);  
  data[0] += 1;
  nRF24L01_WriteReg(RF_RX_ADDR_P5,data,1);
  
  //Write payload width for each pipe
  nRF24L01_WriteReg(RF_RX_PW_P0,&rx_pw[0],1);
  nRF24L01_WriteReg(RF_RX_PW_P1,&rx_pw[1],1);
  nRF24L01_WriteReg(RF_RX_PW_P2,&rx_pw[2],1);
  nRF24L01_WriteReg(RF_RX_PW_P3,&rx_pw[3],1);
  nRF24L01_WriteReg(RF_RX_PW_P4,&rx_pw[4],1);
  nRF24L01_WriteReg(RF_RX_PW_P5,&rx_pw[5],1);
}

/*******************************************************************************
* Function: nRF24L01_SetTxAddress(UINT8 *tx_addr, UINT8 addr_width)             
*                                                                              
* Variables:                                                                   
* *tx_addr -> Sets the TX_ADDR                                                 
* addr_width -> Sets the address width; Can be from 3 - 5 bytes                
*                                                                              
* Description:                                                                 
* This function will set the TX_ADDR register. Whichever pipe you are writing  
* to on your PRX will match this address. If (tx_addr == NULL), the default    
* TX_ADDR will be used.                                                        
*******************************************************************************/
void nRF24L01_SetTxAddress(UINT8 *tx_addr, UINT8 addr_width)
{
  UINT8 data[5];
  
  //Set TX_ADDR to user specified address
  if (tx_addr != NULL)  
    nRF24L01_WriteReg(RF_TX_ADDR,tx_addr,addr_width); 
  
  //Set TX_ADDR to default address (0xE7E7E7E7E7)
  else
  {
    data[0] = RF_TX_ADDR_B0_DV;
    data[1] = RF_TX_ADDR_B1_DV;
    data[2] = RF_TX_ADDR_B2_DV;
    data[3] = RF_TX_ADDR_B3_DV;
    data[4] = RF_TX_ADDR_B4_DV;
    
    nRF24L01_WriteReg(RF_TX_ADDR,data,addr_width); 
  }  
}

/*******************************************************************************
* Function: nRF24L01_Set_Tx_ACK_Address(UINT8 *addr, UINT8 addr_width)          
*                                                                              
* Variables:                                                                   
* *addr -> Sets the TX_ADDR and RX_ADDR_P0 to the same address (Needed for ACK) 
* addr_width -> Sets the address width; Can be from 3 - 5 bytes                
*                                                                              
* Description:                                                                 
* This function will set TX_ADDR and RX_ADDR_P0 to the same address. This is   
* needed in order to receive ACKs from multiple data pipes when using enhanced 
* shockburst. The ACK is always received on RX_ADDR_P0 and must match the same 
* address as TX_ADDR.                                                          
*******************************************************************************/
void nRF24L01_Set_Tx_ACK_Address(UINT8 *addr, UINT8 addr_width)
{
  UINT8 data[5];
  
  //Set TX_ADDR and RX_ADDR_P0 to user specified address
  if (addr != NULL)  
  {
    nRF24L01_WriteReg(RF_TX_ADDR,addr,addr_width); 
    nRF24L01_WriteReg(RF_RX_ADDR_P0,addr,addr_width); 
  }  
    
  //Set TX_ADDR and RX_ADDR_P0 to default address (0xE7E7E7E7E7)
  else
  {
    data[0] = RF_TX_ADDR_B0_DV;
    data[1] = RF_TX_ADDR_B1_DV;
    data[2] = RF_TX_ADDR_B2_DV;
    data[3] = RF_TX_ADDR_B3_DV;
    data[4] = RF_TX_ADDR_B4_DV;
    
    nRF24L01_WriteReg(RF_TX_ADDR,data,addr_width); 
    nRF24L01_WriteReg(RF_RX_ADDR_P0,data,addr_width); 
  } 
}

/*******************************************************************************
* Function: nRF24L01_Transmit(void)                                            
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will pulse RF_CE high for 10us which will transmit the pload.  
*******************************************************************************/
void nRF24L01_Transmit(void)
{
 RF_CE = 1;
 Delay_us(10);
 RF_CE = 0;
} 

/*******************************************************************************
* Function: nRF24L01_Tx(UINT8 *data, UINT8 len)                                 
*                                                                              
* Variables:                                                                   
* *data -> Data that is to be loaded into TX FIFO and transmitted              
* len -> The amount of data to be loaded into the TX FIFO                      
*                                                                              
* Description:                                                                 
* This function will load data into the TX FIFO and transmit it to a receiver. 
*******************************************************************************/
void nRF24L01_Tx(UINT8 *data, UINT8 len)
{
  UINT8 status;
 
  //Read STATUS
  nRF24L01_ReadReg(RF_STATUS,&status,1);  
  
  //Make sure TX FIFO is not full before loading in more data
  if (status & RF_STATUS_TX_FULL) 
    nRF24L01_CMD(RF_FLUSH_TX); 
  
  //Clear IRQ interrupts
  status = 0x7E;
  nRF24L01_WriteReg(RF_STATUS,&status,1);
  
  //Load payload into TX FIFO and transmit it
  nRF24L01_WriteReg(RF_WR_TX_PLOAD,data,len);
  nRF24L01_Transmit();
}

/*******************************************************************************
* Function: nRF24L01_Tx_SB(UINT8 *data, UINT8 len)                              
*                                                                              
* Variables:                                                                   
* *data -> Data that is to be loaded into TX FIFO and transmitted              
* len -> The amount of data to be loaded into the TX FIFO                      
*                                                                              
* Description:                                                                 
* This function will load data into the TX FIFO and transmit it to a receiver. 
* This routine is only to be used when Enhanced/Shockburst is set up. It will  
* check for an ACK response and auto-retransmit the data for a specified       
* amount of retries until the data is received by the PRX or the function      
* times out and returns an error.                                              
*******************************************************************************/
UINT8 nRF24L01_Tx_SB(UINT8 *data, UINT8 len)
{
  UINT8 status;
  UINT8 review;
  UINT8 retry = 0;
   
  //Read STATUS
  nRF24L01_ReadReg(RF_STATUS,&status,1);  
  
  //Make sure TX FIFO is not full before loading in more data
  while (status & RF_STATUS_TX_FULL)   
       nRF24L01_CMD(RF_FLUSH_TX);
 
//  //Clear IRQ interrupts
//  status = 0x7E;
//  nRF24L01_WriteReg(RF_STATUS,&status,1);
  
  //Load payload into TX FIFO and transmit it
  nRF24L01_WriteReg(RF_WR_TX_PLOAD,data,len);
  nRF24L01_Transmit();
  
  //Check for ACK packet to ensure a successful transmission
  review = nRF24L01_Check_ACK();
  
  //If no ACK, re-transmit data up to 10x more times manually (The auto-retransmit executes as well)
  while (review)
  {
    nRF24L01_CMD(RF_REUSE_TX_PL);
    nRF24L01_Transmit();
    
    //If no ACK is received and the loop times-out, send back value from 'review' to diagnose error
    //Be sure to flush tx and any unsent data in TX FIFO
    if (retry >= 10)
    {
      nRF24L01_WriteReg(RF_FLUSH_TX,NULL,0);
      return review;
    }  
    
    //Check again for ACK packet to ensure a successful transmission  
    review = nRF24L01_Check_ACK();
    retry++;
  }
  
  //If this line executes, it means (review == 0) which means a successful transmission
  return review;   
} 

/*******************************************************************************
* Function: nRF24L01_WriteReg(UINT8 reg, UINT8 *data, UINT8 len)                 
*                                                                              
* Variables:                                                                   
* reg -> Contains the register address that is to be written to                
* *data -> Pointer to data that is to be written to 'reg'                      
* len -> The amount of data to write pointed to by '*data'                     
*                                                                              
* Description:                                                                 
* This function will write a user specified amount of data to the specified    
* register in the nRF24L01+. The function will return the STATUS register.     
*******************************************************************************/
void nRF24L01_WriteReg(UINT8 reg, UINT8 *data, UINT8 len)
{
  UINT8 i;
  
  RF_CSN = 0; //Enable nRF24L01+
  Delay_us(2);
  
  SPI_Send(RF_WRITE_REG | reg); //Send WRITE command and register address
  Delay_us(2);
  
  //Begin writing specified amount of data
  for (i = 0;i < len;i++)
  {
    SPI_Send(data[i]);
  	Delay_us(2);
  }
    
  RF_CSN = 1; //Disable nRF24L01+
  Delay_us(2);
}

#endif
